"""
Python module generated from Java source file org.bukkit.DyeColor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import ImmutableMap
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class DyeColor(Enum):
    """
    All supported color values for dyes and cloth
    """

    WHITE = (0x0, 0xF, Color.fromRGB(0xF9FFFE), Color.fromRGB(0xF0F0F0))
    """
    Represents white dye.
    """
    ORANGE = (0x1, 0xE, Color.fromRGB(0xF9801D), Color.fromRGB(0xEB8844))
    """
    Represents orange dye.
    """
    MAGENTA = (0x2, 0xD, Color.fromRGB(0xC74EBD), Color.fromRGB(0xC354CD))
    """
    Represents magenta dye.
    """
    LIGHT_BLUE = (0x3, 0xC, Color.fromRGB(0x3AB3DA), Color.fromRGB(0x6689D3))
    """
    Represents light blue dye.
    """
    YELLOW = (0x4, 0xB, Color.fromRGB(0xFED83D), Color.fromRGB(0xDECF2A))
    """
    Represents yellow dye.
    """
    LIME = (0x5, 0xA, Color.fromRGB(0x80C71F), Color.fromRGB(0x41CD34))
    """
    Represents lime dye.
    """
    PINK = (0x6, 0x9, Color.fromRGB(0xF38BAA), Color.fromRGB(0xD88198))
    """
    Represents pink dye.
    """
    GRAY = (0x7, 0x8, Color.fromRGB(0x474F52), Color.fromRGB(0x434343))
    """
    Represents gray dye.
    """
    LIGHT_GRAY = (0x8, 0x7, Color.fromRGB(0x9D9D97), Color.fromRGB(0xABABAB))
    """
    Represents light gray dye.
    """
    CYAN = (0x9, 0x6, Color.fromRGB(0x169C9C), Color.fromRGB(0x287697))
    """
    Represents cyan dye.
    """
    PURPLE = (0xA, 0x5, Color.fromRGB(0x8932B8), Color.fromRGB(0x7B2FBE))
    """
    Represents purple dye.
    """
    BLUE = (0xB, 0x4, Color.fromRGB(0x3C44AA), Color.fromRGB(0x253192))
    """
    Represents blue dye.
    """
    BROWN = (0xC, 0x3, Color.fromRGB(0x835432), Color.fromRGB(0x51301A))
    """
    Represents brown dye.
    """
    GREEN = (0xD, 0x2, Color.fromRGB(0x5E7C16), Color.fromRGB(0x3B511A))
    """
    Represents green dye.
    """
    RED = (0xE, 0x1, Color.fromRGB(0xB02E26), Color.fromRGB(0xB3312C))
    """
    Represents red dye.
    """
    BLACK = (0xF, 0x0, Color.fromRGB(0x1D1D21), Color.fromRGB(0x1E1B1B))
    """
    Represents black dye.
    """


    def getWoolData(self) -> int:
        """
        Gets the associated wool data value representing this color.

        Returns
        - A byte containing the wool data value of this color

        See
        - .getDyeData()

        Deprecated
        - Magic value
        """
        ...


    def getDyeData(self) -> int:
        """
        Gets the associated dye data value representing this color.

        Returns
        - A byte containing the dye data value of this color

        See
        - .getWoolData()

        Deprecated
        - Magic value
        """
        ...


    def getColor(self) -> "Color":
        """
        Gets the color that this dye represents.

        Returns
        - The Color that this dye represents
        """
        ...


    def getFireworkColor(self) -> "Color":
        """
        Gets the firework color that this dye represents.

        Returns
        - The Color that this dye represents
        """
        ...


    @staticmethod
    def getByWoolData(data: int) -> "DyeColor":
        """
        Gets the DyeColor with the given wool data value.

        Arguments
        - data: Wool data value to fetch

        Returns
        - The DyeColor representing the given value, or null if
            it doesn't exist

        See
        - .getByDyeData(byte)

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByDyeData(data: int) -> "DyeColor":
        """
        Gets the DyeColor with the given dye data value.

        Arguments
        - data: Dye data value to fetch

        Returns
        - The DyeColor representing the given value, or null if
            it doesn't exist

        See
        - .getByWoolData(byte)

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByColor(color: "Color") -> "DyeColor":
        """
        Gets the DyeColor with the given color value.

        Arguments
        - color: Color value to get the dye by

        Returns
        - The DyeColor representing the given value, or null if
            it doesn't exist
        """
        ...


    @staticmethod
    def getByFireworkColor(color: "Color") -> "DyeColor":
        """
        Gets the DyeColor with the given firework color value.

        Arguments
        - color: Color value to get dye by

        Returns
        - The DyeColor representing the given value, or null if
            it doesn't exist
        """
        ...


    @staticmethod
    def legacyValueOf(name: str) -> "DyeColor":
        """
        Gets the DyeColor for the given name, possibly doing legacy transformations.

        Arguments
        - name: dye name

        Returns
        - dye color

        Deprecated
        - legacy use only
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockDamageAbortEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockDamageAbortEvent(BlockEvent):
    """
    Called when a player stops damaging a Block.

    See
    - BlockDamageEvent
    """

    def __init__(self, player: "Player", block: "Block", itemInHand: "ItemStack"):
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the player that stopped damaging the block involved in this event.

        Returns
        - The player that stopped damaging the block
        """
        ...


    def getItemInHand(self) -> "ItemStack":
        """
        Gets the ItemStack for the item currently in the player's hand.

        Returns
        - The ItemStack for the item currently in the player's hand
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



from .PluginMessageListener import PluginMessageListener
from .ReservedChannelException import ReservedChannelException
from .PluginMessageRecipient import PluginMessageRecipient
from .MessageTooLargeException import MessageTooLargeException
from .ChannelNameTooLongException import ChannelNameTooLongException
from .ChannelNotRegisteredException import ChannelNotRegisteredException
from .StandardMessenger import StandardMessenger
from .PluginChannelDirection import PluginChannelDirection
from .PluginMessageListenerRegistration import PluginMessageListenerRegistration
from .Messenger import Messenger


"""
Python module generated from Java source file org.bukkit.enchantments.EnchantmentWrapper

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.enchantments import *
from typing import Any, Callable, Iterable, Tuple


class EnchantmentWrapper(Enchantment):
    """
    A simple wrapper for ease of selecting Enchantments

    Deprecated
    - only for backwards compatibility, EnchantmentWrapper is no longer used.
    """

    def getEnchantment(self) -> "Enchantment":
        """
        Gets the enchantment bound to this wrapper

        Returns
        - Enchantment
        """
        ...



"""
Python module generated from Java source file com.google.common.base.Ascii

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from java.nio.charset import StandardCharsets
from typing import Any, Callable, Iterable, Tuple


class Ascii:
    """
    Static methods pertaining to ASCII characters (those in the range of values `0x00` through
    `0x7F`), and to strings containing such characters.
    
    ASCII utilities also exist in other classes of this package:
    
    
      <!-- TODO(kevinb): how can we make this not produce a warning when building gwt javadoc? -->
      - StandardCharsets.US_ASCII specifies the `Charset` of ASCII characters.
      - CharMatcher.ascii matches ASCII characters and provides text processing methods
          which operate only on the ASCII characters of a string.

    Author(s)
    - Gregory Kick

    Since
    - 7.0
    """

    NUL = 0
    """
    Null ('\0'): The all-zeros character which may serve to accomplish time fill and media fill.
    Normally used as a C string terminator.
    
    Although RFC 20 names this as "Null", note that it is distinct from the C/C++ "NULL"
    pointer.

    Since
    - 8.0
    """
    SOH = 1
    """
    Start of Heading: A communication control character used at the beginning of a sequence of
    characters which constitute a machine-sensible address or routing information. Such a sequence
    is referred to as the "heading." An STX character has the effect of terminating a heading.

    Since
    - 8.0
    """
    STX = 2
    """
    Start of Text: A communication control character which precedes a sequence of characters that
    is to be treated as an entity and entirely transmitted through to the ultimate destination.
    Such a sequence is referred to as "text." STX may be used to terminate a sequence of characters
    started by SOH.

    Since
    - 8.0
    """
    ETX = 3
    """
    End of Text: A communication control character used to terminate a sequence of characters
    started with STX and transmitted as an entity.

    Since
    - 8.0
    """
    EOT = 4
    """
    End of Transmission: A communication control character used to indicate the conclusion of a
    transmission, which may have contained one or more texts and any associated headings.

    Since
    - 8.0
    """
    ENQ = 5
    """
    Enquiry: A communication control character used in data communication systems as a request for
    a response from a remote station. It may be used as a "Who Are You" (WRU) to obtain
    identification, or may be used to obtain station status, or both.

    Since
    - 8.0
    """
    ACK = 6
    """
    Acknowledge: A communication control character transmitted by a receiver as an affirmative
    response to a sender.

    Since
    - 8.0
    """
    BEL = 7
    """
    Bell ('\a'): A character for use when there is a need to call for human attention. It may
    control alarm or attention devices.

    Since
    - 8.0
    """
    BS = 8
    """
    Backspace ('\b'): A format effector which controls the movement of the printing position one
    printing space backward on the same printing line. (Applicable also to display devices.)

    Since
    - 8.0
    """
    HT = 9
    """
    Horizontal Tabulation ('\t'): A format effector which controls the movement of the printing
    position to the next in a series of predetermined positions along the printing line.
    (Applicable also to display devices and the skip function on punched cards.)

    Since
    - 8.0
    """
    LF = 10
    """
    Line Feed ('\n'): A format effector which controls the movement of the printing position to the
    next printing line. (Applicable also to display devices.) Where appropriate, this character may
    have the meaning "New Line" (NL), a format effector which controls the movement of the printing
    point to the first printing position on the next printing line. Use of this convention requires
    agreement between sender and recipient of data.

    Since
    - 8.0
    """
    NL = 10
    """
    Alternate name for .LF. (`LF` is preferred.)

    Since
    - 8.0
    """
    VT = 11
    """
    Vertical Tabulation ('\v'): A format effector which controls the movement of the printing
    position to the next in a series of predetermined printing lines. (Applicable also to display
    devices.)

    Since
    - 8.0
    """
    FF = 12
    """
    Form Feed ('\f'): A format effector which controls the movement of the printing position to the
    first pre-determined printing line on the next form or page. (Applicable also to display
    devices.)

    Since
    - 8.0
    """
    CR = 13
    """
    Carriage Return ('\r'): A format effector which controls the movement of the printing position
    to the first printing position on the same printing line. (Applicable also to display devices.)

    Since
    - 8.0
    """
    SO = 14
    """
    Shift Out: A control character indicating that the code combinations which follow shall be
    interpreted as outside of the character set of the standard code table until a Shift In
    character is reached.

    Since
    - 8.0
    """
    SI = 15
    """
    Shift In: A control character indicating that the code combinations which follow shall be
    interpreted according to the standard code table.

    Since
    - 8.0
    """
    DLE = 16
    """
    Data Link Escape: A communication control character which will change the meaning of a limited
    number of contiguously following characters. It is used exclusively to provide supplementary
    controls in data communication networks.

    Since
    - 8.0
    """
    DC1 = 17
    XON = 17
    DC2 = 18
    """
    Device Control 2. Characters for the control of ancillary devices associated with data
    processing or telecommunication systems, more especially switching devices "on" or "off." (If a
    single "stop" control is required to interrupt or turn off ancillary devices, DC4 is the
    preferred assignment.)

    Since
    - 8.0
    """
    DC3 = 19
    XOFF = 19
    DC4 = 20
    """
    Device Control 4. Characters for the control of ancillary devices associated with data
    processing or telecommunication systems, more especially switching devices "on" or "off." (If a
    single "stop" control is required to interrupt or turn off ancillary devices, DC4 is the
    preferred assignment.)

    Since
    - 8.0
    """
    NAK = 21
    """
    Negative Acknowledge: A communication control character transmitted by a receiver as a negative
    response to the sender.

    Since
    - 8.0
    """
    SYN = 22
    """
    Synchronous Idle: A communication control character used by a synchronous transmission system
    in the absence of any other character to provide a signal from which synchronism may be
    achieved or retained.

    Since
    - 8.0
    """
    ETB = 23
    """
    End of Transmission Block: A communication control character used to indicate the end of a
    block of data for communication purposes. ETB is used for blocking data where the block
    structure is not necessarily related to the processing format.

    Since
    - 8.0
    """
    CAN = 24
    """
    Cancel: A control character used to indicate that the data with which it is sent is in error or
    is to be disregarded.

    Since
    - 8.0
    """
    EM = 25
    """
    End of Medium: A control character associated with the sent data which may be used to identify
    the physical end of the medium, or the end of the used, or wanted, portion of information
    recorded on a medium. (The position of this character does not necessarily correspond to the
    physical end of the medium.)

    Since
    - 8.0
    """
    SUB = 26
    """
    Substitute: A character that may be substituted for a character which is determined to be
    invalid or in error.

    Since
    - 8.0
    """
    ESC = 27
    """
    Escape: A control character intended to provide code extension (supplementary characters) in
    general information interchange. The Escape character itself is a prefix affecting the
    interpretation of a limited number of contiguously following characters.

    Since
    - 8.0
    """
    FS = 28
    """
    File Separator: These four information separators may be used within data in optional fashion,
    except that their hierarchical relationship shall be: FS is the most inclusive, then GS, then
    RS, and US is least inclusive. (The content and length of a File, Group, Record, or Unit are
    not specified.)

    Since
    - 8.0
    """
    GS = 29
    """
    Group Separator: These four information separators may be used within data in optional fashion,
    except that their hierarchical relationship shall be: FS is the most inclusive, then GS, then
    RS, and US is least inclusive. (The content and length of a File, Group, Record, or Unit are
    not specified.)

    Since
    - 8.0
    """
    RS = 30
    """
    Record Separator: These four information separators may be used within data in optional
    fashion, except that their hierarchical relationship shall be: FS is the most inclusive, then
    GS, then RS, and US is least inclusive. (The content and length of a File, Group, Record, or
    Unit are not specified.)

    Since
    - 8.0
    """
    US = 31
    """
    Unit Separator: These four information separators may be used within data in optional fashion,
    except that their hierarchical relationship shall be: FS is the most inclusive, then GS, then
    RS, and US is least inclusive. (The content and length of a File, Group, Record, or Unit are
    not specified.)

    Since
    - 8.0
    """
    SP = 32
    """
    Space: A normally non-printing graphic character used to separate words. It is also a format
    effector which controls the movement of the printing position, one printing position forward.
    (Applicable also to display devices.)

    Since
    - 8.0
    """
    SPACE = 32
    """
    Alternate name for .SP.

    Since
    - 8.0
    """
    DEL = 127
    """
    Delete: This character is used primarily to "erase" or "obliterate" erroneous or unwanted
    characters in perforated tape.

    Since
    - 8.0
    """
    MIN = 0
    """
    The minimum value of an ASCII character.

    Since
    - 9.0 (was type `int` before 12.0)
    """
    MAX = 127
    """
    The maximum value of an ASCII character.

    Since
    - 9.0 (was type `int` before 12.0)
    """


    @staticmethod
    def toLowerCase(string: str) -> str:
        """
        Returns a copy of the input string in which all .isUpperCase(char) uppercase ASCII
        characters have been converted to lowercase. All other characters are copied without
        modification.
        """
        ...


    @staticmethod
    def toLowerCase(chars: "CharSequence") -> str:
        """
        Returns a copy of the input character sequence in which all .isUpperCase(char)
        uppercase ASCII characters have been converted to lowercase. All other characters are copied
        without modification.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def toLowerCase(c: str) -> str:
        """
        If the argument is an .isUpperCase(char) uppercase ASCII character, returns the
        lowercase equivalent. Otherwise returns the argument.
        """
        ...


    @staticmethod
    def toUpperCase(string: str) -> str:
        """
        Returns a copy of the input string in which all .isLowerCase(char) lowercase ASCII
        characters have been converted to uppercase. All other characters are copied without
        modification.
        """
        ...


    @staticmethod
    def toUpperCase(chars: "CharSequence") -> str:
        """
        Returns a copy of the input character sequence in which all .isLowerCase(char)
        lowercase ASCII characters have been converted to uppercase. All other characters are copied
        without modification.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def toUpperCase(c: str) -> str:
        """
        If the argument is a .isLowerCase(char) lowercase ASCII character, returns the
        uppercase equivalent. Otherwise returns the argument.
        """
        ...


    @staticmethod
    def isLowerCase(c: str) -> bool:
        """
        Indicates whether `c` is one of the twenty-six lowercase ASCII alphabetic characters
        between `'a'` and `'z'` inclusive. All others (including non-ASCII characters)
        return `False`.
        """
        ...


    @staticmethod
    def isUpperCase(c: str) -> bool:
        """
        Indicates whether `c` is one of the twenty-six uppercase ASCII alphabetic characters
        between `'A'` and `'Z'` inclusive. All others (including non-ASCII characters)
        return `False`.
        """
        ...


    @staticmethod
    def truncate(seq: "CharSequence", maxLength: int, truncationIndicator: str) -> str:
        """
        Truncates the given character sequence to the given maximum length. If the length of the
        sequence is greater than `maxLength`, the returned string will be exactly `maxLength` chars in length and will end with the given `truncationIndicator`. Otherwise,
        the sequence will be returned as a string with no changes to the content.
        
        Examples:
        
        ````Ascii.truncate("foobar", 7, "..."); // returns "foobar"
        Ascii.truncate("foobar", 5, "..."); // returns "fo..."````
        
        **Note:** This method *may* work with certain non-ASCII text but is not safe for use
        with arbitrary Unicode text. It is mostly intended for use with text that is known to be safe
        for use with it (such as all-ASCII text) and for simple debugging text. When using this method,
        consider the following:
        
        
          - it may split surrogate pairs
          - it may split characters and combining characters
          - it does not consider word boundaries
          - if truncating for display to users, there are other considerations that must be taken
              into account
          - the appropriate truncation indicator may be locale-dependent
          - it is safe to use non-ASCII characters in the truncation indicator

        Raises
        - IllegalArgumentException: if `maxLength` is less than the length of `truncationIndicator`

        Since
        - 16.0
        """
        ...


    @staticmethod
    def equalsIgnoreCase(s1: "CharSequence", s2: "CharSequence") -> bool:
        """
        Indicates whether the contents of the given character sequences `s1` and `s2` are
        equal, ignoring the case of any ASCII alphabetic characters between `'a'` and `'z'`
        or `'A'` and `'Z'` inclusive.
        
        This method is significantly faster than String.equalsIgnoreCase and should be used
        in preference if at least one of the parameters is known to contain only ASCII characters.
        
        Note however that this method does not always behave identically to expressions such as:
        
        
          - `string.toUpperCase().equals("UPPER CASE ASCII")`
          - `string.toLowerCase().equals("lower case ascii")`
        
        
        due to case-folding of some non-ASCII characters (which does not occur in String.equalsIgnoreCase). However in almost all cases that ASCII strings are used, the author
        probably wanted the behavior provided by this method rather than the subtle and sometimes
        surprising behavior of `toUpperCase()` and `toLowerCase()`.

        Since
        - 16.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.ChatColor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Maps
from enum import Enum
from java.util.regex import Pattern
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class ChatColor(Enum):
    """
    All supported color values for chat
    """

# Static fields
    COLOR_CHAR = '\u00A7'
    """
    The special character which prefixes all chat colour codes. Use this if
    you need to dynamically convert colour codes from your custom format.
    """


    BLACK = ('0', 0x00)
    """
    Represents black
    """
    DARK_BLUE = ('1', 0x1)
    """
    Represents dark blue
    """
    DARK_GREEN = ('2', 0x2)
    """
    Represents dark green
    """
    DARK_AQUA = ('3', 0x3)
    """
    Represents dark blue (aqua)
    """
    DARK_RED = ('4', 0x4)
    """
    Represents dark red
    """
    DARK_PURPLE = ('5', 0x5)
    """
    Represents dark purple
    """
    GOLD = ('6', 0x6)
    """
    Represents gold
    """
    GRAY = ('7', 0x7)
    """
    Represents gray
    """
    DARK_GRAY = ('8', 0x8)
    """
    Represents dark gray
    """
    BLUE = ('9', 0x9)
    """
    Represents blue
    """
    GREEN = ('a', 0xA)
    """
    Represents green
    """
    AQUA = ('b', 0xB)
    """
    Represents aqua
    """
    RED = ('c', 0xC)
    """
    Represents red
    """
    LIGHT_PURPLE = ('d', 0xD)
    """
    Represents light purple
    """
    YELLOW = ('e', 0xE)
    """
    Represents yellow
    """
    WHITE = ('f', 0xF)
    """
    Represents white
    """
    MAGIC = ('k', 0x10, True)
    """
    Represents magical characters that change around randomly
    """
    BOLD = ('l', 0x11, True)
    """
    Makes the text bold.
    """
    STRIKETHROUGH = ('m', 0x12, True)
    """
    Makes a line appear through the text.
    """
    UNDERLINE = ('n', 0x13, True)
    """
    Makes the text appear underlined.
    """
    ITALIC = ('o', 0x14, True)
    """
    Makes the text italic.
    """
    RESET = ('r', 0x15)
    """
    Resets all previous chat colors or formats.
    """


    def asBungee(self) -> "net.md_5.bungee.api.ChatColor":
        ...


    def getChar(self) -> str:
        """
        Gets the char value associated with this color

        Returns
        - A char value of this color code
        """
        ...


    def toString(self) -> str:
        ...


    def isFormat(self) -> bool:
        """
        Checks if this code is a format code as opposed to a color code.

        Returns
        - whether this ChatColor is a format code
        """
        ...


    def isColor(self) -> bool:
        """
        Checks if this code is a color code as opposed to a format code.

        Returns
        - whether this ChatColor is a color code
        """
        ...


    @staticmethod
    def getByChar(code: str) -> "ChatColor":
        """
        Gets the color represented by the specified color code

        Arguments
        - code: Code to check

        Returns
        - Associative org.bukkit.ChatColor with the given code,
            or null if it doesn't exist
        """
        ...


    @staticmethod
    def getByChar(code: str) -> "ChatColor":
        """
        Gets the color represented by the specified color code

        Arguments
        - code: Code to check

        Returns
        - Associative org.bukkit.ChatColor with the given code,
            or null if it doesn't exist
        """
        ...


    @staticmethod
    def stripColor(input: str) -> str:
        """
        Strips the given message of all color codes

        Arguments
        - input: String to strip of color

        Returns
        - A copy of the input string, without any coloring
        """
        ...


    @staticmethod
    def translateAlternateColorCodes(altColorChar: str, textToTranslate: str) -> str:
        """
        Translates a string using an alternate color code character into a
        string that uses the internal ChatColor.COLOR_CODE color code
        character. The alternate color code character will only be replaced if
        it is immediately followed by 0-9, A-F, a-f, K-O, k-o, R or r.

        Arguments
        - altColorChar: The alternate color code character to replace. Ex: &
        - textToTranslate: Text containing the alternate color code character.

        Returns
        - Text containing the ChatColor.COLOR_CODE color code character.
        """
        ...


    @staticmethod
    def getLastColors(input: str) -> str:
        """
        Gets the ChatColors used at the end of the given input string.

        Arguments
        - input: Input string to retrieve the colors from.

        Returns
        - Any remaining ChatColors to pass onto the next line.
        """
        ...



from setuptools import setup, find_namespace_packages

setup(
    name='spigot-sources',
    version='1.21.4',
    author='magicmq',
    author_email='business@magicmq.dev',
    description='Translated Spigot sources for writing PySpigot scripts',
    url='https://pyspigot-docs.magicmq.dev',
    py_modules=[],
    packages=find_namespace_packages(),
    python_requires='>=3.6',
    classifiers = [
        'Programming Language :: Python :: 3',
        'Natural Language :: English',
        'Topic :: Software Development :: Libraries :: Java Libraries',
        'License :: OSI Approved :: Apache Software License'
    ],
)



"""
Python module generated from Java source file org.bukkit.tag.DamageTypeTags

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Bukkit
from org.bukkit import NamespacedKey
from org.bukkit import Tag
from org.bukkit.damage import DamageType
from org.bukkit.tag import *
from typing import Any, Callable, Iterable, Tuple


class DamageTypeTags:
    """
    Vanilla DamageType Tag tags.
    """

    DAMAGES_HELMET = getTag("damages_helmet")
    """
    Vanilla tag representing damage types which damage helmets.
    """
    BYPASSES_ARMOR = getTag("bypasses_armor")
    """
    Vanilla tag representing damage types which bypass armor.
    """
    BYPASSES_SHIELD = getTag("bypasses_shield")
    """
    Vanilla tag representing damage types which bypass shields.
    """
    BYPASSES_INVULNERABILITY = getTag("bypasses_invulnerability")
    """
    Vanilla tag representing damage types which bypass invulnerability.
    """
    BYPASSES_COOLDOWN = getTag("bypasses_cooldown")
    """
    Vanilla tag representing damage types which bypass cooldowns.
    
    **Note:** this can be null unless a datapack add values to this tag because vanilla not has any values for this.
    """
    BYPASSES_EFFECTS = getTag("bypasses_effects")
    """
    Vanilla tag representing damage types which bypass effects.
    """
    BYPASSES_RESISTANCE = getTag("bypasses_resistance")
    """
    Vanilla tag representing damage types which bypass resistance.
    """
    BYPASSES_ENCHANTMENTS = getTag("bypasses_enchantments")
    """
    Vanilla tag representing damage types which bypass enchantments.
    """
    IS_FIRE = getTag("is_fire")
    """
    Vanilla tag representing all fire damage types.
    """
    IS_PROJECTILE = getTag("is_projectile")
    """
    Vanilla tag representing damage types which originate from projectiles.
    """
    WITCH_RESISTANT_TO = getTag("witch_resistant_to")
    """
    Vanilla tag representing damage types which witches are resistant to.
    """
    IS_EXPLOSION = getTag("is_explosion")
    """
    Vanilla tag representing all explosion damage types.
    """
    IS_FALL = getTag("is_fall")
    """
    Vanilla tag representing all fall damage types.
    """
    IS_DROWNING = getTag("is_drowning")
    """
    Vanilla tag representing all drowning damage types.
    """
    IS_FREEZING = getTag("is_freezing")
    """
    Vanilla tag representing all freezing damage types.
    """
    IS_LIGHTNING = getTag("is_lightning")
    """
    Vanilla tag representing all lightning damage types.
    """
    NO_ANGER = getTag("no_anger")
    """
    Vanilla tag representing damage types which do not cause entities to
    anger.
    """
    NO_IMPACT = getTag("no_impact")
    """
    Vanilla tag representing damage types which do not cause an impact.
    """
    ALWAYS_MOST_SIGNIFICANT_FALL = getTag("always_most_significant_fall")
    """
    Vanilla tag representing damage types which cause maximum fall damage.
    """
    WITHER_IMMUNE_TO = getTag("wither_immune_to")
    """
    Vanilla tag representing damage types which withers are immune to.
    """
    IGNITES_ARMOR_STANDS = getTag("ignites_armor_stands")
    """
    Vanilla tag representing damage types which ignite armor stands.
    """
    BURNS_ARMOR_STANDS = getTag("burns_armor_stands")
    """
    Vanilla tag representing damage types which burn armor stands.
    """
    AVOIDS_GUARDIAN_THORNS = getTag("avoids_guardian_thorns")
    """
    Vanilla tag representing damage types which avoid guardian thorn damage.
    """
    ALWAYS_TRIGGERS_SILVERFISH = getTag("always_triggers_silverfish")
    """
    Vanilla tag representing damage types which always trigger silverfish.
    """
    ALWAYS_HURTS_ENDER_DRAGONS = getTag("always_hurts_ender_dragons")
    """
    Vanilla tag representing damage types which always hurt enderdragons.
    """
    NO_KNOCKBACK = getTag("no_knockback")
    """
    Vanilla tag representing damage types which do not cause knockback.
    """
    ALWAYS_KILLS_ARMOR_STANDS = getTag("always_kills_armor_stands")
    """
    Vanilla tag representing damage types which always kill armor stands.
    """
    CAN_BREAK_ARMOR_STAND = getTag("can_break_armor_stand")
    """
    Vanilla tag representing damage types which can break armor stands.
    """
    BYPASSES_WOLF_ARMOR = getTag("bypasses_wolf_armor")
    """
    Vanilla tag representing damage types which bypass wolf armor.
    """
    IS_PLAYER_ATTACK = getTag("is_player_attack")
    """
    Vanilla tag representing damage types which are from player attacks.
    """
    BURN_FROM_STEPPING = getTag("burn_from_stepping")
    """
    Vanilla tag representing damage types which originate from hot blocks.
    """
    PANIC_CAUSES = getTag("panic_causes")
    """
    Vanilla tag representing damage types which cause entities to panic.
    """
    PANIC_ENVIRONMENTAL_CAUSES = getTag("panic_environmental_causes")
    """
    Vanilla tag representing environmental damage types which cause entities
    to panic.
    """
    IS_MACE_SMASH = getTag("mace_smash")
    """
    Vanilla tag representing damage types which originate from mace smashes.
    """
    REGISTRY_DAMAGE_TYPES = "damage_types"
    """
    Internal use only.
    """



"""
Python module generated from Java source file java.util.concurrent.BlockingDeque

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Deque
from java.util import Iterator
from java.util import NoSuchElementException
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class BlockingDeque(BlockingQueue, Deque):
    """
    A Deque that additionally supports blocking operations that wait
    for the deque to become non-empty when retrieving an element, and wait for
    space to become available in the deque when storing an element.
    
    `BlockingDeque` methods come in four forms, with different ways
    of handling operations that cannot be satisfied immediately, but may be
    satisfied at some point in the future:
    one throws an exception, the second returns a special value (either
    `null` or `False`, depending on the operation), the third
    blocks the current thread indefinitely until the operation can succeed,
    and the fourth blocks for only a given maximum time limit before giving
    up.  These methods are summarized in the following table:
    
    <table class="plain">
    <caption>Summary of BlockingDeque methods</caption>
     <tr>
       <th id="First" colspan="5"> First Element (Head)</th>
     </tr>
     <tr>
       <td></td>
       <th id="FThrow" style="font-weight:normal; font-style: italic">Throws exception</th>
       <th id="FValue" style="font-weight:normal; font-style: italic">Special value</th>
       <th id="FBlock" style="font-weight:normal; font-style: italic">Blocks</th>
       <th id="FTimes" style="font-weight:normal; font-style: italic">Times out</th>
     </tr>
     <tr>
       <th id="FInsert" style="text-align:left">Insert</th>
       <td headers="First FInsert FThrow">.addFirst(Object) addFirst(e)</td>
       <td headers="First FInsert FValue">.offerFirst(Object) offerFirst(e)</td>
       <td headers="First FInsert FBlock">.putFirst(Object) putFirst(e)</td>
       <td headers="First FInsert FTimes">.offerFirst(Object, long, TimeUnit) offerFirst(e, time, unit)</td>
     </tr>
     <tr>
       <th id="FRemove" style="text-align:left">Remove</th>
       <td headers="First FRemove FThrow">.removeFirst() removeFirst()</td>
       <td headers="First FRemove FValue">.pollFirst() pollFirst()</td>
       <td headers="First FRemove FBlock">.takeFirst() takeFirst()</td>
       <td headers="First FRemove FTimes">.pollFirst(long, TimeUnit) pollFirst(time, unit)</td>
     </tr>
     <tr>
       <th id="FExamine" style="text-align:left">Examine</th>
       <td headers="First FExamine FThrow">.getFirst() getFirst()</td>
       <td headers="First FExamine FValue">.peekFirst() peekFirst()</td>
       <td headers="First FExamine FBlock" style="font-style:italic">not applicable</td>
       <td headers="First FExamine FTimes" style="font-style:italic">not applicable</td>
     </tr>
     <tr>
       <th id="Last" colspan="5"> Last Element (Tail)</th>
     </tr>
     <tr>
       <td></td>
       <th id="LThrow" style="font-weight:normal; font-style: italic">Throws exception</th>
       <th id="LValue" style="font-weight:normal; font-style: italic">Special value</th>
       <th id="LBlock" style="font-weight:normal; font-style: italic">Blocks</th>
       <th id="LTimes" style="font-weight:normal; font-style: italic">Times out</th>
     </tr>
     <tr>
       <th id="LInsert" style="text-align:left">Insert</th>
       <td headers="Last LInsert LThrow">.addLast(Object) addLast(e)</td>
       <td headers="Last LInsert LValue">.offerLast(Object) offerLast(e)</td>
       <td headers="Last LInsert LBlock">.putLast(Object) putLast(e)</td>
       <td headers="Last LInsert LTimes">.offerLast(Object, long, TimeUnit) offerLast(e, time, unit)</td>
     </tr>
     <tr>
       <th id="LRemove" style="text-align:left">Remove</th>
       <td headers="Last LRemove LThrow">.removeLast() removeLast()</td>
       <td headers="Last LRemove LValue">.pollLast() pollLast()</td>
       <td headers="Last LRemove LBlock">.takeLast() takeLast()</td>
       <td headers="Last LRemove LTimes">.pollLast(long, TimeUnit) pollLast(time, unit)</td>
     </tr>
     <tr>
       <th id="LExamine" style="text-align:left">Examine</th>
       <td headers="Last LExamine LThrow">.getLast() getLast()</td>
       <td headers="Last LExamine LValue">.peekLast() peekLast()</td>
       <td headers="Last LExamine LBlock" style="font-style:italic">not applicable</td>
       <td headers="Last LExamine LTimes" style="font-style:italic">not applicable</td>
     </tr>
    </table>
    
    Like any BlockingQueue, a `BlockingDeque` is thread safe,
    does not permit null elements, and may (or may not) be
    capacity-constrained.
    
    A `BlockingDeque` implementation may be used directly as a FIFO
    `BlockingQueue`. The methods inherited from the
    `BlockingQueue` interface are precisely equivalent to
    `BlockingDeque` methods as indicated in the following table:
    
    <table class="plain">
    <caption>Comparison of BlockingQueue and BlockingDeque methods</caption>
     <tr>
       <td></td>
       <th id="BQueue"> `BlockingQueue` Method</th>
       <th id="BDeque"> Equivalent `BlockingDeque` Method</th>
     </tr>
     <tr>
       <th id="Insert" rowspan="4" style="text-align:left; vertical-align:top">Insert</th>
       <th id="add" style="font-weight:normal; text-align:left">.add(Object) add(e)</th>
       <td headers="Insert BDeque add">.addLast(Object) addLast(e)</td>
     </tr>
     <tr>
       <th id="offer1" style="font-weight:normal; text-align:left">.offer(Object) offer(e)</th>
       <td headers="Insert BDeque offer1">.offerLast(Object) offerLast(e)</td>
     </tr>
     <tr>
       <th id="put" style="font-weight:normal; text-align:left">.put(Object) put(e)</th>
       <td headers="Insert BDeque put">.putLast(Object) putLast(e)</td>
     </tr>
     <tr>
       <th id="offer2" style="font-weight:normal; text-align:left">.offer(Object, long, TimeUnit) offer(e, time, unit)</th>
       <td headers="Insert BDeque offer2">.offerLast(Object, long, TimeUnit) offerLast(e, time, unit)</td>
     </tr>
     <tr>
       <th id="Remove" rowspan="4" style="text-align:left; vertical-align:top">Remove</th>
       <th id="remove" style="font-weight:normal; text-align:left">.remove() remove()</th>
       <td headers="Remove BDeque remove">.removeFirst() removeFirst()</td>
     </tr>
     <tr>
       <th id="poll1" style="font-weight:normal; text-align:left">.poll() poll()</th>
       <td headers="Remove BDeque poll1">.pollFirst() pollFirst()</td>
     </tr>
     <tr>
       <th id="take" style="font-weight:normal; text-align:left">.take() take()</th>
       <td headers="Remove BDeque take">.takeFirst() takeFirst()</td>
     </tr>
     <tr>
       <th id="poll2" style="font-weight:normal; text-align:left">.poll(long, TimeUnit) poll(time, unit)</th>
       <td headers="Remove BDeque poll2">.pollFirst(long, TimeUnit) pollFirst(time, unit)</td>
     </tr>
     <tr>
       <th id="Examine" rowspan="2" style="text-align:left; vertical-align:top">Examine</th>
       <th id="element" style="font-weight:normal; text-align:left">.element() element()</th>
       <td headers="Examine BDeque element">.getFirst() getFirst()</td>
     </tr>
     <tr>
       <th id="peek" style="font-weight:normal; text-align:left">.peek() peek()</th>
       <td headers="Examine BDeque peek">.peekFirst() peekFirst()</td>
     </tr>
    </table>
    
    Memory consistency effects: As with other concurrent
    collections, actions in a thread prior to placing an object into a
    `BlockingDeque`
    <a href="package-summary.html#MemoryVisibility">*happen-before*</a>
    actions subsequent to the access or removal of that element from
    the `BlockingDeque` in another thread.
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements held in this deque

    Author(s)
    - Doug Lea

    Since
    - 1.6
    """

    def addFirst(self, e: "E") -> None:
        """
        Inserts the specified element at the front of this deque if it is
        possible to do so immediately without violating capacity restrictions,
        throwing an `IllegalStateException` if no space is currently
        available.  When using a capacity-restricted deque, it is generally
        preferable to use .offerFirst(Object) offerFirst.

        Arguments
        - e: the element to add

        Raises
        - IllegalStateException: 
        - ClassCastException: 
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: 
        """
        ...


    def addLast(self, e: "E") -> None:
        """
        Inserts the specified element at the end of this deque if it is
        possible to do so immediately without violating capacity restrictions,
        throwing an `IllegalStateException` if no space is currently
        available.  When using a capacity-restricted deque, it is generally
        preferable to use .offerLast(Object) offerLast.

        Arguments
        - e: the element to add

        Raises
        - IllegalStateException: 
        - ClassCastException: 
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: 
        """
        ...


    def offerFirst(self, e: "E") -> bool:
        """
        Inserts the specified element at the front of this deque if it is
        possible to do so immediately without violating capacity restrictions,
        returning `True` upon success and `False` if no space is
        currently available.
        When using a capacity-restricted deque, this method is generally
        preferable to the .addFirst(Object) addFirst method, which can
        fail to insert an element only by throwing an exception.

        Arguments
        - e: the element to add

        Raises
        - ClassCastException: 
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: 
        """
        ...


    def offerLast(self, e: "E") -> bool:
        """
        Inserts the specified element at the end of this deque if it is
        possible to do so immediately without violating capacity restrictions,
        returning `True` upon success and `False` if no space is
        currently available.
        When using a capacity-restricted deque, this method is generally
        preferable to the .addLast(Object) addLast method, which can
        fail to insert an element only by throwing an exception.

        Arguments
        - e: the element to add

        Raises
        - ClassCastException: 
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: 
        """
        ...


    def putFirst(self, e: "E") -> None:
        """
        Inserts the specified element at the front of this deque,
        waiting if necessary for space to become available.

        Arguments
        - e: the element to add

        Raises
        - InterruptedException: if interrupted while waiting
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def putLast(self, e: "E") -> None:
        """
        Inserts the specified element at the end of this deque,
        waiting if necessary for space to become available.

        Arguments
        - e: the element to add

        Raises
        - InterruptedException: if interrupted while waiting
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offerFirst(self, e: "E", timeout: int, unit: "TimeUnit") -> bool:
        """
        Inserts the specified element at the front of this deque,
        waiting up to the specified wait time if necessary for space to
        become available.

        Arguments
        - e: the element to add
        - timeout: how long to wait before giving up, in units of
               `unit`
        - unit: a `TimeUnit` determining how to interpret the
               `timeout` parameter

        Returns
        - `True` if successful, or `False` if
                the specified waiting time elapses before space is available

        Raises
        - InterruptedException: if interrupted while waiting
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offerLast(self, e: "E", timeout: int, unit: "TimeUnit") -> bool:
        """
        Inserts the specified element at the end of this deque,
        waiting up to the specified wait time if necessary for space to
        become available.

        Arguments
        - e: the element to add
        - timeout: how long to wait before giving up, in units of
               `unit`
        - unit: a `TimeUnit` determining how to interpret the
               `timeout` parameter

        Returns
        - `True` if successful, or `False` if
                the specified waiting time elapses before space is available

        Raises
        - InterruptedException: if interrupted while waiting
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def takeFirst(self) -> "E":
        """
        Retrieves and removes the first element of this deque, waiting
        if necessary until an element becomes available.

        Returns
        - the head of this deque

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def takeLast(self) -> "E":
        """
        Retrieves and removes the last element of this deque, waiting
        if necessary until an element becomes available.

        Returns
        - the tail of this deque

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def pollFirst(self, timeout: int, unit: "TimeUnit") -> "E":
        """
        Retrieves and removes the first element of this deque, waiting
        up to the specified wait time if necessary for an element to
        become available.

        Arguments
        - timeout: how long to wait before giving up, in units of
               `unit`
        - unit: a `TimeUnit` determining how to interpret the
               `timeout` parameter

        Returns
        - the head of this deque, or `null` if the specified
                waiting time elapses before an element is available

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def pollLast(self, timeout: int, unit: "TimeUnit") -> "E":
        """
        Retrieves and removes the last element of this deque, waiting
        up to the specified wait time if necessary for an element to
        become available.

        Arguments
        - timeout: how long to wait before giving up, in units of
               `unit`
        - unit: a `TimeUnit` determining how to interpret the
               `timeout` parameter

        Returns
        - the tail of this deque, or `null` if the specified
                waiting time elapses before an element is available

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def removeFirstOccurrence(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if an element was removed as a result of this call

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def removeLastOccurrence(self, o: "Object") -> bool:
        """
        Removes the last occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the last element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if an element was removed as a result of this call

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def add(self, e: "E") -> bool:
        """
        Inserts the specified element into the queue represented by this deque
        (in other words, at the tail of this deque) if it is possible to do so
        immediately without violating capacity restrictions, returning
        `True` upon success and throwing an
        `IllegalStateException` if no space is currently available.
        When using a capacity-restricted deque, it is generally preferable to
        use .offer(Object) offer.
        
        This method is equivalent to .addLast(Object) addLast.

        Arguments
        - e: the element to add

        Raises
        - IllegalStateException: 
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offer(self, e: "E") -> bool:
        """
        Inserts the specified element into the queue represented by this deque
        (in other words, at the tail of this deque) if it is possible to do so
        immediately without violating capacity restrictions, returning
        `True` upon success and `False` if no space is currently
        available.  When using a capacity-restricted deque, this method is
        generally preferable to the .add method, which can fail to
        insert an element only by throwing an exception.
        
        This method is equivalent to .offerLast(Object) offerLast.

        Arguments
        - e: the element to add

        Raises
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def put(self, e: "E") -> None:
        """
        Inserts the specified element into the queue represented by this deque
        (in other words, at the tail of this deque), waiting if necessary for
        space to become available.
        
        This method is equivalent to .putLast(Object) putLast.

        Arguments
        - e: the element to add

        Raises
        - InterruptedException: 
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def offer(self, e: "E", timeout: int, unit: "TimeUnit") -> bool:
        """
        Inserts the specified element into the queue represented by this deque
        (in other words, at the tail of this deque), waiting up to the
        specified wait time if necessary for space to become available.
        
        This method is equivalent to
        .offerLast(Object,long,TimeUnit) offerLast.

        Arguments
        - e: the element to add

        Returns
        - `True` if the element was added to this deque, else
                `False`

        Raises
        - InterruptedException: 
        - ClassCastException: if the class of the specified element
                prevents it from being added to this deque
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: if some property of the specified
                element prevents it from being added to this deque
        """
        ...


    def remove(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque).
        This method differs from .poll() poll() only in that it
        throws an exception if this deque is empty.
        
        This method is equivalent to .removeFirst() removeFirst.

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def poll(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque), or returns
        `null` if this deque is empty.
        
        This method is equivalent to .pollFirst().

        Returns
        - the head of this deque, or `null` if this deque is empty
        """
        ...


    def take(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque), waiting if
        necessary until an element becomes available.
        
        This method is equivalent to .takeFirst() takeFirst.

        Returns
        - the head of this deque

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def poll(self, timeout: int, unit: "TimeUnit") -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque), waiting up to the
        specified wait time if necessary for an element to become available.
        
        This method is equivalent to
        .pollFirst(long,TimeUnit) pollFirst.

        Returns
        - the head of this deque, or `null` if the
                specified waiting time elapses before an element is available

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    def element(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque (in other words, the first element of this deque).
        This method differs from .peek() peek only in that it throws an
        exception if this deque is empty.
        
        This method is equivalent to .getFirst() getFirst.

        Returns
        - the head of this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def peek(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque (in other words, the first element of this deque), or
        returns `null` if this deque is empty.
        
        This method is equivalent to .peekFirst() peekFirst.

        Returns
        - the head of this deque, or `null` if this deque is empty
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).
        
        This method is equivalent to
        .removeFirstOccurrence(Object) removeFirstOccurrence.

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if this deque changed as a result of the call

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Returns `True` if this deque contains the specified element.
        More formally, returns `True` if and only if this deque contains
        at least one element `e` such that `o.equals(e)`.

        Arguments
        - o: object to be checked for containment in this deque

        Returns
        - `True` if this deque contains the specified element

        Raises
        - ClassCastException: if the class of the specified element
                is incompatible with this deque
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if the specified element is null
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in this deque.

        Returns
        - the number of elements in this deque
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this deque in proper sequence.
        The elements will be returned in order from first (head) to last (tail).

        Returns
        - an iterator over the elements in this deque in proper sequence
        """
        ...


    def push(self, e: "E") -> None:
        """
        Pushes an element onto the stack represented by this deque (in other
        words, at the head of this deque) if it is possible to do so
        immediately without violating capacity restrictions, throwing an
        `IllegalStateException` if no space is currently available.
        
        This method is equivalent to .addFirst(Object) addFirst.

        Raises
        - IllegalStateException: 
        - ClassCastException: 
        - NullPointerException: if the specified element is null
        - IllegalArgumentException: 
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.FixedSetPrompt

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Joiner
from java.util import Arrays
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class FixedSetPrompt(ValidatingPrompt):
    """
    FixedSetPrompt is the base class for any prompt that requires a fixed set
    response from the user.
    """

    def __init__(self, *fixedSet: Tuple[str, ...]):
        """
        Creates a FixedSetPrompt from a set of strings.
        
        foo = new FixedSetPrompt("bar", "cheese", "panda");

        Arguments
        - fixedSet: A fixed set of strings, one of which the user must
            type.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerBucketFishEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit import Warning
from org.bukkit.entity import Fish
from org.bukkit.entity import Player
from org.bukkit.event.player import *
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class PlayerBucketFishEvent(PlayerBucketEntityEvent):
    """
    This event is called whenever a player attempts to put a fish in a bucket.

    Deprecated
    - Use the more generic PlayerBucketEntityEvent
    """

    def __init__(self, player: "Player", fish: "Fish", waterBucket: "ItemStack", fishBucket: "ItemStack", hand: "EquipmentSlot"):
        ...


    def getEntity(self) -> "Fish":
        """
        Gets the fish involved with this event.

        Returns
        - The fish involved with this event
        """
        ...


    def getWaterBucket(self) -> "ItemStack":
        """
        Gets the bucket used.
        
        This refers to the bucket clicked with, ie Material.WATER_BUCKET.

        Returns
        - The used bucket

        Deprecated
        - Use .getOriginalBucket()
        """
        ...


    def getFishBucket(self) -> "ItemStack":
        """
        Gets the bucket that the fish will be put into.
        
        This refers to the bucket with the fish, ie
        Material.PUFFERFISH_BUCKET.

        Returns
        - The bucket that the fish will be put into

        Deprecated
        - Use .getEntityBucket()
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.Traverser

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.collect import AbstractIterator
from com.google.common.collect import ImmutableSet
from com.google.common.graph import *
from com.google.errorprone.annotations import DoNotMock
from java.util import ArrayDeque
from java.util import Deque
from java.util import Iterator
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Traverser:
    """
    An object that can traverse the nodes that are reachable from a specified (set of) start node(s)
    using a specified SuccessorsFunction.
    
    There are two entry points for creating a `Traverser`: .forTree(SuccessorsFunction) and .forGraph(SuccessorsFunction). You should choose one
    based on your answers to the following questions:
    
    <ol>
      - Is there only one path to any node that's reachable from any start node? (If so, the graph
          to be traversed is a tree or forest even if it is a subgraph of a graph which is neither.)
      - Are the node objects' implementations of `equals()`/`hashCode()` <a
          href="https://github.com/google/guava/wiki/GraphsExplained#non-recursiveness">recursive</a>?
    </ol>
    
    If your answers are:
    
    
      - (1) "no" and (2) "no", use .forGraph(SuccessorsFunction).
      - (1) "yes" and (2) "yes", use .forTree(SuccessorsFunction).
      - (1) "yes" and (2) "no", you can use either, but `forTree()` will be more efficient.
      - (1) "no" and (2) "yes", ***neither will work***, but if you transform your node
          objects into a non-recursive form, you can use `forGraph()`.
    
    
    Type `<N>`: Node parameter type

    Author(s)
    - Jens Nyman

    Since
    - 23.1
    """

    @staticmethod
    def forGraph(graph: "SuccessorsFunction"["N"]) -> "Traverser"["N"]:
        """
        Creates a new traverser for the given general `graph`.
        
        Traversers created using this method are guaranteed to visit each node reachable from the
        start node(s) at most once.
        
        If you know that no node in `graph` is reachable by more than one path from the start
        node(s), consider using .forTree(SuccessorsFunction) instead.
        
        **Performance notes**
        
        
          - Traversals require *O(n)* time (where *n* is the number of nodes reachable from
              the start node), assuming that the node objects have *O(1)* `equals()` and
              `hashCode()` implementations. (See the <a
              href="https://github.com/google/guava/wiki/GraphsExplained#elements-must-be-useable-as-map-keys">
              notes on element objects</a> for more information.)
          - While traversing, the traverser will use *O(n)* space (where *n* is the number
              of nodes that have thus far been visited), plus *O(H)* space (where *H* is the
              number of nodes that have been seen but not yet visited, that is, the "horizon").

        Arguments
        - graph: SuccessorsFunction representing a general graph that may have cycles.
        """
        ...


    @staticmethod
    def forTree(tree: "SuccessorsFunction"["N"]) -> "Traverser"["N"]:
        """
        Creates a new traverser for a directed acyclic graph that has at most one path from the start
        node(s) to any node reachable from the start node(s), and has no paths from any start node to
        any other start node, such as a tree or forest.
        
        `forTree()` is especially useful (versus `forGraph()`) in cases where the data
        structure being traversed is, in addition to being a tree/forest, also defined <a
        href="https://github.com/google/guava/wiki/GraphsExplained#non-recursiveness">recursively</a>.
        This is because the `forTree()`-based implementations don't keep track of visited nodes,
        and therefore don't need to call `equals()` or `hashCode()` on the node objects; this saves
        both time and space versus traversing the same graph using `forGraph()`.
        
        Providing a graph to be traversed for which there is more than one path from the start
        node(s) to any node may lead to:
        
        
          - Traversal not terminating (if the graph has cycles)
          - Nodes being visited multiple times (if multiple paths exist from any start node to any
              node reachable from any start node)
        
        
        **Performance notes**
        
        
          - Traversals require *O(n)* time (where *n* is the number of nodes reachable from
              the start node).
          - While traversing, the traverser will use *O(H)* space (where *H* is the number
              of nodes that have been seen but not yet visited, that is, the "horizon").
        
        
        **Examples** (all edges are directed facing downwards)
        
        The graph below would be valid input with start nodes of `a, f, c`. However, if `b` were *also* a start node, then there would be multiple paths to reach `e` and
        `h`.
        
        ````a     b      c
          / \   / \     |
         /   \ /   \    |
        d     e     f   g
              |
              |
              h````
        
        .
        
        The graph below would be a valid input with start nodes of `a, f`. However, if `b` were a start node, there would be multiple paths to `f`.
        
        ````a     b
          / \   / \
         /   \ /   \
        c     d     e
               \   /
                \ /
                 f````
        
        **Note on binary trees**
        
        This method can be used to traverse over a binary tree. Given methods `leftChild(node)` and `rightChild(node)`, this method can be called as
        
        ````Traverser.forTree(node -> ImmutableList.of(leftChild(node), rightChild(node)));````

        Arguments
        - tree: SuccessorsFunction representing a directed acyclic graph that has at most
            one path between any two nodes
        """
        ...


    def breadthFirst(self, startNode: "N") -> Iterable["N"]:
        """
        Returns an unmodifiable `Iterable` over the nodes reachable from `startNode`, in
        the order of a breadth-first traversal. That is, all the nodes of depth 0 are returned, then
        depth 1, then 2, and so on.
        
        **Example:** The following graph with `startNode` `a` would return nodes in
        the order `abcdef` (assuming successors are returned in alphabetical order).
        
        ````b ---- a ---- d
        |      |
        |      |
        e ---- c ---- f````
        
        The behavior of this method is undefined if the nodes, or the topology of the graph, change
        while iteration is in progress.
        
        The returned `Iterable` can be iterated over multiple times. Every iterator will
        compute its next element on the fly. It is thus possible to limit the traversal to a certain
        number of nodes as follows:
        
        ````Iterables.limit(Traverser.forGraph(graph).breadthFirst(node), maxNumberOfNodes);````
        
        See <a href="https://en.wikipedia.org/wiki/Breadth-first_search">Wikipedia</a> for more
        info.

        Raises
        - IllegalArgumentException: if `startNode` is not an element of the graph
        """
        ...


    def breadthFirst(self, startNodes: Iterable["N"]) -> Iterable["N"]:
        """
        Returns an unmodifiable `Iterable` over the nodes reachable from any of the `startNodes`, in the order of a breadth-first traversal. This is equivalent to a breadth-first
        traversal of a graph with an additional root node whose successors are the listed `startNodes`.

        Raises
        - IllegalArgumentException: if any of `startNodes` is not an element of the graph

        See
        - .breadthFirst(Object)

        Since
        - 24.1
        """
        ...


    def depthFirstPreOrder(self, startNode: "N") -> Iterable["N"]:
        """
        Returns an unmodifiable `Iterable` over the nodes reachable from `startNode`, in
        the order of a depth-first pre-order traversal. "Pre-order" implies that nodes appear in the
        `Iterable` in the order in which they are first visited.
        
        **Example:** The following graph with `startNode` `a` would return nodes in
        the order `abecfd` (assuming successors are returned in alphabetical order).
        
        ````b ---- a ---- d
        |      |
        |      |
        e ---- c ---- f````
        
        The behavior of this method is undefined if the nodes, or the topology of the graph, change
        while iteration is in progress.
        
        The returned `Iterable` can be iterated over multiple times. Every iterator will
        compute its next element on the fly. It is thus possible to limit the traversal to a certain
        number of nodes as follows:
        
        ````Iterables.limit(
            Traverser.forGraph(graph).depthFirstPreOrder(node), maxNumberOfNodes);````
        
        See <a href="https://en.wikipedia.org/wiki/Depth-first_search">Wikipedia</a> for more info.

        Raises
        - IllegalArgumentException: if `startNode` is not an element of the graph
        """
        ...


    def depthFirstPreOrder(self, startNodes: Iterable["N"]) -> Iterable["N"]:
        """
        Returns an unmodifiable `Iterable` over the nodes reachable from any of the `startNodes`, in the order of a depth-first pre-order traversal. This is equivalent to a
        depth-first pre-order traversal of a graph with an additional root node whose successors are
        the listed `startNodes`.

        Raises
        - IllegalArgumentException: if any of `startNodes` is not an element of the graph

        See
        - .depthFirstPreOrder(Object)

        Since
        - 24.1
        """
        ...


    def depthFirstPostOrder(self, startNode: "N") -> Iterable["N"]:
        """
        Returns an unmodifiable `Iterable` over the nodes reachable from `startNode`, in
        the order of a depth-first post-order traversal. "Post-order" implies that nodes appear in the
        `Iterable` in the order in which they are visited for the last time.
        
        **Example:** The following graph with `startNode` `a` would return nodes in
        the order `fcebda` (assuming successors are returned in alphabetical order).
        
        ````b ---- a ---- d
        |      |
        |      |
        e ---- c ---- f````
        
        The behavior of this method is undefined if the nodes, or the topology of the graph, change
        while iteration is in progress.
        
        The returned `Iterable` can be iterated over multiple times. Every iterator will
        compute its next element on the fly. It is thus possible to limit the traversal to a certain
        number of nodes as follows:
        
        ````Iterables.limit(
            Traverser.forGraph(graph).depthFirstPostOrder(node), maxNumberOfNodes);````
        
        See <a href="https://en.wikipedia.org/wiki/Depth-first_search">Wikipedia</a> for more info.

        Raises
        - IllegalArgumentException: if `startNode` is not an element of the graph
        """
        ...


    def depthFirstPostOrder(self, startNodes: Iterable["N"]) -> Iterable["N"]:
        """
        Returns an unmodifiable `Iterable` over the nodes reachable from any of the `startNodes`, in the order of a depth-first post-order traversal. This is equivalent to a
        depth-first post-order traversal of a graph with an additional root node whose successors are
        the listed `startNodes`.

        Raises
        - IllegalArgumentException: if any of `startNodes` is not an element of the graph

        See
        - .depthFirstPostOrder(Object)

        Since
        - 24.1
        """
        ...



"""
Python module generated from Java source file com.google.common.cache.AbstractCache

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.cache import *
from com.google.common.collect import ImmutableMap
from com.google.common.collect import Maps
from java.util.concurrent import Callable
from java.util.concurrent import ConcurrentMap
from java.util.concurrent import ExecutionException
from typing import Any, Callable, Iterable, Tuple


class AbstractCache(Cache):
    """
    This class provides a skeletal implementation of the `Cache` interface to minimize the
    effort required to implement this interface.
    
    To implement a cache, the programmer needs only to extend this class and provide an
    implementation for the .put and .getIfPresent methods. .getAllPresent is
    implemented in terms of .getIfPresent; .putAll is implemented in terms of .put, .invalidateAll(Iterable) is implemented in terms of .invalidate. The
    method .cleanUp is a no-op. All other methods throw an UnsupportedOperationException.

    Author(s)
    - Charles Fry

    Since
    - 10.0
    """

    def get(self, key: "K", valueLoader: "Callable"["V"]) -> "V":
        """
        Since
        - 11.0
        """
        ...


    def getAllPresent(self, keys: Iterable["Object"]) -> "ImmutableMap"["K", "V"]:
        ...


    def put(self, key: "K", value: "V") -> None:
        """
        Since
        - 11.0
        """
        ...


    def putAll(self, m: dict["K", "V"]) -> None:
        """
        Since
        - 12.0
        """
        ...


    def cleanUp(self) -> None:
        ...


    def size(self) -> int:
        ...


    def invalidate(self, key: "Object") -> None:
        ...


    def invalidateAll(self, keys: Iterable["Object"]) -> None:
        """
        Since
        - 11.0
        """
        ...


    def invalidateAll(self) -> None:
        ...


    def stats(self) -> "CacheStats":
        ...


    def asMap(self) -> "ConcurrentMap"["K", "V"]:
        ...


    class StatsCounter:
        """
        Accumulates statistics during the operation of a Cache for presentation by Cache.stats. This is solely intended for consumption by `Cache` implementors.

        Since
        - 10.0
        """

        def recordHits(self, count: int) -> None:
            """
            Records cache hits. This should be called when a cache request returns a cached value.

            Arguments
            - count: the number of hits to record

            Since
            - 11.0
            """
            ...


        def recordMisses(self, count: int) -> None:
            """
            Records cache misses. This should be called when a cache request returns a value that was not
            found in the cache. This method should be called by the loading thread, as well as by threads
            blocking on the load. Multiple concurrent calls to Cache lookup methods with the same
            key on an absent value should result in a single call to either `recordLoadSuccess` or
            `recordLoadException` and multiple calls to this method, despite all being served by
            the results of a single load operation.

            Arguments
            - count: the number of misses to record

            Since
            - 11.0
            """
            ...


        def recordLoadSuccess(self, loadTime: int) -> None:
            """
            Records the successful load of a new entry. This should be called when a cache request causes
            an entry to be loaded, and the loading completes successfully. In contrast to .recordMisses, this method should only be called by the loading thread.

            Arguments
            - loadTime: the number of nanoseconds the cache spent computing or retrieving the new
                value
            """
            ...


        def recordLoadException(self, loadTime: int) -> None:
            """
            Records the failed load of a new entry. This should be called when a cache request causes an
            entry to be loaded, but an exception is thrown while loading the entry. In contrast to .recordMisses, this method should only be called by the loading thread.

            Arguments
            - loadTime: the number of nanoseconds the cache spent computing or retrieving the new
                value prior to an exception being thrown
            """
            ...


        def recordEviction(self) -> None:
            """
            Records the eviction of an entry from the cache. This should only been called when an entry
            is evicted due to the cache's eviction strategy, and not as a result of manual Cache.invalidate invalidations.
            """
            ...


        def snapshot(self) -> "CacheStats":
            """
            Returns a snapshot of this counter's values. Note that this may be an inconsistent view, as
            it may be interleaved with update operations.
            """
            ...


    class SimpleStatsCounter(StatsCounter):
        """
        A thread-safe StatsCounter implementation for use by Cache implementors.

        Since
        - 10.0
        """

        def __init__(self):
            """
            Constructs an instance with all counts initialized to zero.
            """
            ...


        def recordHits(self, count: int) -> None:
            """
            Since
            - 11.0
            """
            ...


        def recordMisses(self, count: int) -> None:
            """
            Since
            - 11.0
            """
            ...


        def recordLoadSuccess(self, loadTime: int) -> None:
            ...


        def recordLoadException(self, loadTime: int) -> None:
            ...


        def recordEviction(self) -> None:
            ...


        def snapshot(self) -> "CacheStats":
            ...


        def incrementBy(self, other: "StatsCounter") -> None:
            """
            Increments all counters by the values in `other`.
            """
            ...



"""
Python module generated from Java source file com.google.common.collect.HashMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Preconditions
from com.google.common.collect import *
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class HashMultimap(HashMultimapGwtSerializationDependencies):
    """
    Implementation of Multimap using hash tables.
    
    The multimap does not store duplicate key-value pairs. Adding a new key-value pair equal to an
    existing key-value pair has no effect.
    
    Keys and values may be null. All optional multimap methods are supported, and all returned
    views are modifiable.
    
    This class is not threadsafe when any concurrent operations update the multimap. Concurrent
    read operations will work correctly if the last write *happens-before* any reads. To allow
    concurrent update operations, wrap your multimap with a call to Multimaps.synchronizedSetMultimap.
    
    **Warning:** Do not modify either a key *or a value* of a `HashMultimap` in a
    way that affects its Object.equals behavior. Undefined behavior and bugs will result.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def create() -> "HashMultimap"["K", "V"]:
        """
        Creates a new, empty `HashMultimap` with the default initial capacities.
        
        You may also consider the equivalent `MultimapBuilder.hashKeys().hashSetValues().build()`, which provides more control over the
        underlying data structure.
        """
        ...


    @staticmethod
    def create(expectedKeys: int, expectedValuesPerKey: int) -> "HashMultimap"["K", "V"]:
        """
        Constructs an empty `HashMultimap` with enough capacity to hold the specified numbers of
        keys and values without rehashing.
        
        You may also consider the equivalent `MultimapBuilder.hashKeys(expectedKeys).hashSetValues(expectedValuesPerKey).build()`, which
        provides more control over the underlying data structure.

        Arguments
        - expectedKeys: the expected number of distinct keys
        - expectedValuesPerKey: the expected average number of values per key

        Raises
        - IllegalArgumentException: if `expectedKeys` or `expectedValuesPerKey` is
            negative
        """
        ...


    @staticmethod
    def create(multimap: "Multimap"["K", "V"]) -> "HashMultimap"["K", "V"]:
        """
        Constructs a `HashMultimap` with the same mappings as the specified multimap. If a
        key-value mapping appears multiple times in the input multimap, it only appears once in the
        constructed multimap.
        
        You may also consider the equivalent `MultimapBuilder.hashKeys().hashSetValues().build(multimap)`, which provides more control over
        the underlying data structure.

        Arguments
        - multimap: the multimap whose contents are copied to this multimap
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.StandardRowSortedTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Supplier
from com.google.common.collect import *
from com.google.j2objc.annotations import WeakOuter
from java.util import Comparator
from java.util import SortedMap
from java.util import SortedSet
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class StandardRowSortedTable(StandardTable, RowSortedTable):
    """
    Implementation of `Table` whose iteration ordering across row keys is sorted by their
    natural ordering or by a supplied comparator. Note that iterations across the columns keys for a
    single row key may or may not be ordered, depending on the implementation. When rows and columns
    are both sorted, it's easier to use the TreeBasedTable subclass.
    
    The .rowKeySet method returns a SortedSet and the .rowMap method
    returns a SortedMap, instead of the Set and Map specified by the Table interface.
    
    Null keys and values are not supported.
    
    See the StandardTable superclass for more information about the behavior of this
    class.

    Author(s)
    - Jared Levy
    """

    def rowKeySet(self) -> "SortedSet"["R"]:
        """
        
        
        This method returns a SortedSet, instead of the `Set` specified in the Table interface.
        """
        ...


    def rowMap(self) -> "SortedMap"["R", dict["C", "V"]]:
        """
        
        
        This method returns a SortedMap, instead of the `Map` specified in the Table interface.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.BiMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class BiMap(Map):
    """
    A bimap (or "bidirectional map") is a map that preserves the uniqueness of its values as well as
    that of its keys. This constraint enables bimaps to support an "inverse view", which is another
    bimap containing the same entries as this bimap but with reversed keys and values.
    
    <h3>Implementations</h3>
    
    
      - ImmutableBiMap
      - HashBiMap
      - EnumBiMap
      - EnumHashBiMap
    
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#bimap">`BiMap`</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    def put(self, key: "K", value: "V") -> "V":
        """
        Raises
        - IllegalArgumentException: if the given value is already bound to a different key in this
            bimap. The bimap will remain unmodified in this event. To avoid this exception, call .forcePut instead.
        """
        ...


    def forcePut(self, key: "K", value: "V") -> "V":
        """
        An alternate form of `put` that silently removes any existing entry with the value `value` before proceeding with the .put operation. If the bimap previously contained the
        provided key-value mapping, this method has no effect.
        
        Note that a successful call to this method could cause the size of the bimap to increase by
        one, stay the same, or even decrease by one.
        
        **Warning:** If an existing entry with this value is removed, the key for that entry is
        discarded and not returned.

        Arguments
        - key: the key with which the specified value is to be associated
        - value: the value to be associated with the specified key

        Returns
        - the value that was previously associated with the key, or `null` if there was no
            previous entry. (If the bimap contains null values, then `forcePut`, like `put`, returns `null` both if the key is absent and if it is present with a null
            value.)
        """
        ...


    def putAll(self, map: dict["K", "V"]) -> None:
        """
        
        
        **Warning:** the results of calling this method may vary depending on the iteration order
        of `map`.

        Raises
        - IllegalArgumentException: if an attempt to `put` any entry fails. Note that some
            map entries may have been added to the bimap before the exception was thrown.
        """
        ...


    def values(self) -> set["V"]:
        """
        
        
        Because a bimap has unique values, this method returns a Set, instead of the java.util.Collection specified in the Map interface.
        """
        ...


    def inverse(self) -> "BiMap"["V", "K"]:
        """
        Returns the inverse view of this bimap, which maps each of this bimap's values to its
        associated key. The two bimaps are backed by the same data; any changes to one will appear in
        the other.
        
        **Note:**There is no guaranteed correspondence between the iteration order of a bimap and
        that of its inverse.

        Returns
        - the inverse view of this bimap
        """
        ...



"""
Python module generated from Java source file com.google.common.base.Charsets

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import *
from java.nio.charset import Charset
from java.nio.charset import StandardCharsets
from typing import Any, Callable, Iterable, Tuple


class Charsets:
    """
    Contains constant definitions for the six standard Charset instances, which are
    guaranteed to be supported by all Java platform implementations.
    
    Assuming you're free to choose, note that **.UTF_8 is widely preferred**.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/StringsExplained#charsets">`Charsets`</a>.

    Author(s)
    - Mike Bostock

    Since
    - 1.0
    """

    US_ASCII = StandardCharsets.US_ASCII
    """
    US-ASCII: seven-bit ASCII, the Basic Latin block of the Unicode character set (ISO646-US).
    
    **Note:** this constant is now unnecessary and should be treated as deprecated; use
    StandardCharsets.US_ASCII instead.
    """
    ISO_8859_1 = StandardCharsets.ISO_8859_1
    """
    ISO-8859-1: ISO Latin Alphabet Number 1 (ISO-LATIN-1).
    
    **Note:** this constant is now unnecessary and should be treated as deprecated; use
    StandardCharsets.ISO_8859_1 instead.
    """
    UTF_8 = StandardCharsets.UTF_8
    """
    UTF-8: eight-bit UCS Transformation Format.
    
    **Note:** this constant is now unnecessary and should be treated as deprecated; use
    StandardCharsets.UTF_8 instead.
    """
    UTF_16BE = StandardCharsets.UTF_16BE
    """
    UTF-16BE: sixteen-bit UCS Transformation Format, big-endian byte order.
    
    **Note:** this constant is now unnecessary and should be treated as deprecated; use
    StandardCharsets.UTF_16BE instead.
    """
    UTF_16LE = StandardCharsets.UTF_16LE
    """
    UTF-16LE: sixteen-bit UCS Transformation Format, little-endian byte order.
    
    **Note:** this constant is now unnecessary and should be treated as deprecated; use
    StandardCharsets.UTF_16LE instead.
    """
    UTF_16 = StandardCharsets.UTF_16
    """
    UTF-16: sixteen-bit UCS Transformation Format, byte order identified by an optional byte-order
    mark.
    
    **Note:** this constant is now unnecessary and should be treated as deprecated; use
    StandardCharsets.UTF_16 instead.
    """



"""
Python module generated from Java source file org.bukkit.block.data.type.Stairs

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import Bisected
from org.bukkit.block.data import Directional
from org.bukkit.block.data import Waterlogged
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Stairs(Bisected, Directional, Waterlogged):
    """
    'shape' represents the texture and bounding box shape of these stairs.
    """

    def getShape(self) -> "Shape":
        """
        Gets the value of the 'shape' property.

        Returns
        - the 'shape' value
        """
        ...


    def setShape(self, shape: "Shape") -> None:
        """
        Sets the value of the 'shape' property.

        Arguments
        - shape: the new 'shape' value
        """
        ...


    class Shape(Enum):
        """
        The shape of a stair block - used for constructing corners.
        """

        STRAIGHT = 0
        """
        Regular stair block.
        """
        INNER_LEFT = 1
        """
        Inner corner stair block with higher left side.
        """
        INNER_RIGHT = 2
        """
        Inner corner stair block with higher right side.
        """
        OUTER_LEFT = 3
        """
        Outer corner stair block with higher left side.
        """
        OUTER_RIGHT = 4
        """
        Outer corner stair block with higher right side.
        """



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.consumable.effects.ConsumableApplyEffects

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory.meta.components.consumable.effects import *
from org.bukkit.potion import PotionEffect
from typing import Any, Callable, Iterable, Tuple


class ConsumableApplyEffects(ConsumableEffect):
    """
    Represent the effects applied when an item is consumed.
    """

    def getEffects(self) -> list["PotionEffect"]:
        """
        Gets the effects which may be applied by this item when consumed.

        Returns
        - consumable effects
        """
        ...


    def setEffects(self, effects: list["PotionEffect"]) -> None:
        """
        Sets the effects which may be applied by this item when consumed.

        Arguments
        - effects: new effects
        """
        ...


    def addEffect(self, effect: "PotionEffect") -> "PotionEffect":
        """
        Adds an effect which may be applied by this item when consumed.

        Arguments
        - effect: the effect

        Returns
        - the added effect
        """
        ...


    def getProbability(self) -> float:
        """
        Gets the probability of this effect being applied.

        Returns
        - probability
        """
        ...


    def setProbability(self, probability: float) -> None:
        """
        Sets the probability of this effect being applied.

        Arguments
        - probability: between 0 and 1 inclusive.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockPlaceEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.block import BlockState
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class BlockPlaceEvent(BlockEvent, Cancellable):
    """
    Called when a block is placed by a player.
    
    If a Block Place event is cancelled, the block will not be placed.
    """

    def __init__(self, placedBlock: "Block", replacedBlockState: "BlockState", placedAgainst: "Block", itemInHand: "ItemStack", thePlayer: "Player", canBuild: bool):
        ...


    def __init__(self, placedBlock: "Block", replacedBlockState: "BlockState", placedAgainst: "Block", itemInHand: "ItemStack", thePlayer: "Player", canBuild: bool, hand: "EquipmentSlot"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the player who placed the block involved in this event.

        Returns
        - The Player who placed the block involved in this event
        """
        ...


    def getBlockPlaced(self) -> "Block":
        """
        Clarity method for getting the placed block. Not really needed except
        for reasons of clarity.

        Returns
        - The Block that was placed
        """
        ...


    def getBlockReplacedState(self) -> "BlockState":
        """
        Gets the BlockState for the block which was replaced. Material type air
        mostly.

        Returns
        - The BlockState for the block which was replaced.
        """
        ...


    def getBlockAgainst(self) -> "Block":
        """
        Gets the block that this block was placed against

        Returns
        - Block the block that the new block was placed against
        """
        ...


    def getItemInHand(self) -> "ItemStack":
        """
        Gets the item in the player's hand when they placed the block.

        Returns
        - The ItemStack for the item in the player's hand when they
            placed the block
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Gets the hand which placed the block

        Returns
        - Main or off-hand, depending on which hand was used to place the block
        """
        ...


    def canBuild(self) -> bool:
        """
        Gets the value whether the player would be allowed to build here.
        Defaults to spawn if the server was going to stop them (such as, the
        player is in Spawn). Note that this is an entirely different check
        than BLOCK_CANBUILD, as this refers to a player, not universe-physics
        rule like cactus on dirt.

        Returns
        - boolean whether the server would allow a player to build here
        """
        ...


    def setBuild(self, canBuild: bool) -> None:
        """
        Sets the canBuild state of this event. Set to True if you want the
        player to be able to build.

        Arguments
        - canBuild: True if you want the player to be able to build
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.help.HelpMap

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.help import *
from typing import Any, Callable, Iterable, Tuple


class HelpMap:
    """
    The HelpMap tracks all help topics registered in a Bukkit server. When the
    server starts up or is reloaded, help is processed and topics are added in
    the following order:
    
    <ol>
    - General topics are loaded from the help.yml
    - Plugins load and optionally call `addTopic()`
    - Registered plugin commands are processed by HelpTopicFactory
        objects to create topics
    - Topic contents are amended as directed in help.yml
    </ol>
    """

    def getHelpTopic(self, topicName: str) -> "HelpTopic":
        """
        Returns a help topic for a given topic name.

        Arguments
        - topicName: The help topic name to look up.

        Returns
        - A HelpTopic object matching the topic name or null if
            none can be found.
        """
        ...


    def getHelpTopics(self) -> Iterable["HelpTopic"]:
        """
        Returns a collection of all the registered help topics.

        Returns
        - All the registered help topics.
        """
        ...


    def addTopic(self, topic: "HelpTopic") -> None:
        """
        Adds a topic to the server's help index.

        Arguments
        - topic: The new help topic to add.
        """
        ...


    def clear(self) -> None:
        """
        Clears out the contents of the help index. Normally called during
        server reload.
        """
        ...


    def registerHelpTopicFactory(self, commandClass: type[Any], factory: "HelpTopicFactory"[Any]) -> None:
        """
        Associates a HelpTopicFactory object with given command base
        class. Plugins typically call this method during `onLoad()`. Once
        registered, the custom HelpTopicFactory will be used to create a custom
        HelpTopic for all commands deriving from the `commandClass` base class, or all commands deriving from org.bukkit.command.PluginCommand who's executor derives from `commandClass` base class.

        Arguments
        - commandClass: The class for which the custom HelpTopicFactory
            applies. Must derive from either org.bukkit.command.Command
            or org.bukkit.command.CommandExecutor.
        - factory: The HelpTopicFactory implementation to associate
            with the `commandClass`.

        Raises
        - IllegalArgumentException: Thrown if `commandClass` does
            not derive from a legal base class.
        """
        ...


    def getIgnoredPlugins(self) -> list[str]:
        """
        Gets the list of plugins the server administrator has chosen to exclude
        from the help index. Plugin authors who choose to directly extend
        org.bukkit.command.Command instead of org.bukkit.command.PluginCommand will need to check this collection in
        their HelpTopicFactory implementations to ensure they meet the
        server administrator's expectations.

        Returns
        - A list of plugins that should be excluded from the help index.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.Powerable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import *
from typing import Any, Callable, Iterable, Tuple


class Powerable(BlockData):
    """
    'powered' indicates whether this block is in the powered state or not, i.e.
    receiving a redstone current of power &gt; 0.
    """

    def isPowered(self) -> bool:
        """
        Gets the value of the 'powered' property.

        Returns
        - the 'powered' value
        """
        ...


    def setPowered(self, powered: bool) -> None:
        """
        Sets the value of the 'powered' property.

        Arguments
        - powered: the new 'powered' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.permissions.PermissionRemovedExecutor

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.permissions import *
from typing import Any, Callable, Iterable, Tuple


class PermissionRemovedExecutor:
    """
    Represents a class which is to be notified when a PermissionAttachment is removed from a Permissible
    """

    def attachmentRemoved(self, attachment: "PermissionAttachment") -> None:
        """
        Called when a PermissionAttachment is removed from a Permissible

        Arguments
        - attachment: Attachment which was removed
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Leaves

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit import TreeSpecies
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Leaves(Wood):
    """
    Represents the different types of leaf block that may be permanent or can
    decay when too far from a log.

    See
    - Material.LEGACY_LEAVES_2

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Constructs a leaf block.
        """
        ...


    def __init__(self, species: "TreeSpecies"):
        """
        Constructs a leaf block of the given tree species.

        Arguments
        - species: the species of the wood block
        """
        ...


    def __init__(self, species: "TreeSpecies", isDecayable: bool):
        """
        Constructs a leaf block of the given tree species and flag for whether
        this leaf block will disappear when too far from a log.

        Arguments
        - species: the species of the wood block
        - isDecayable: whether the block is permanent or can disappear
        """
        ...


    def __init__(self, type: "Material"):
        """
        Constructs a leaf block of the given type.

        Arguments
        - type: the type of leaf block
        """
        ...


    def __init__(self, type: "Material", species: "TreeSpecies"):
        """
        Constructs a leaf block of the given type and tree species.

        Arguments
        - type: the type of leaf block
        - species: the species of the wood block
        """
        ...


    def __init__(self, type: "Material", species: "TreeSpecies", isDecayable: bool):
        """
        Constructs a leaf block of the given type and tree species and flag for
        whether this leaf block will disappear when too far from a log.

        Arguments
        - type: the type of leaf block
        - species: the species of the wood block
        - isDecayable: whether the block is permanent or can disappear
        """
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def isDecaying(self) -> bool:
        """
        Checks if this leaf block is in the process of decaying

        Returns
        - True if the leaf block is in the process of decaying
        """
        ...


    def setDecaying(self, isDecaying: bool) -> None:
        """
        Set whether this leaf block is in the process of decaying

        Arguments
        - isDecaying: whether the block is decaying or not
        """
        ...


    def isDecayable(self) -> bool:
        """
        Checks if this leaf block is permanent or can decay when too far from a
        log

        Returns
        - True if the leaf block is permanent or can decay when too far
        from a log
        """
        ...


    def setDecayable(self, isDecayable: bool) -> None:
        """
        Set whether this leaf block will disappear when too far from a log

        Arguments
        - isDecayable: whether the block is permanent or can disappear
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Leaves":
        ...



"""
Python module generated from Java source file org.bukkit.entity.Fox

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Fox(Animals, Sittable):
    """
    What does the fox say?
    """

    def getFoxType(self) -> "Type":
        """
        Gets the current type of this fox.

        Returns
        - Type of the fox.
        """
        ...


    def setFoxType(self, type: "Type") -> None:
        """
        Sets the current type of this fox.

        Arguments
        - type: New type of this fox.
        """
        ...


    def isCrouching(self) -> bool:
        """
        Checks if this animal is crouching

        Returns
        - True if crouching
        """
        ...


    def setCrouching(self, crouching: bool) -> None:
        """
        Sets if this animal is crouching.

        Arguments
        - crouching: True if crouching
        """
        ...


    def setSleeping(self, sleeping: bool) -> None:
        """
        Sets if this animal is sleeping.

        Arguments
        - sleeping: True if sleeping
        """
        ...


    def getFirstTrustedPlayer(self) -> "AnimalTamer":
        """
        Gets the first trusted player.

        Returns
        - the owning AnimalTamer, or null if not owned
        """
        ...


    def setFirstTrustedPlayer(self, player: "AnimalTamer") -> None:
        """
        Set the first trusted player.
        
        The first trusted player may only be removed after the second.

        Arguments
        - player: the AnimalTamer to be trusted
        """
        ...


    def getSecondTrustedPlayer(self) -> "AnimalTamer":
        """
        Gets the second trusted player.

        Returns
        - the owning AnimalTamer, or null if not owned
        """
        ...


    def setSecondTrustedPlayer(self, player: "AnimalTamer") -> None:
        """
        Set the second trusted player.
        
        The second trusted player may only be added after the first.

        Arguments
        - player: the AnimalTamer to be trusted
        """
        ...


    def isFaceplanted(self) -> bool:
        """
        Gets whether the fox is faceplanting the ground

        Returns
        - Whether the fox is faceplanting the ground
        """
        ...


    class Type(Enum):
        """
        Represents the various different fox types there are.
        """

        RED = 0
        SNOW = 1



"""
Python module generated from Java source file com.google.gson.JsonStreamParser

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from com.google.gson.internal import Streams
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonToken
from com.google.gson.stream import MalformedJsonException
from java.io import IOException
from java.io import Reader
from java.io import StringReader
from java.util import Iterator
from java.util import NoSuchElementException
from typing import Any, Callable, Iterable, Tuple


class JsonStreamParser(Iterator):
    """
    A streaming parser that allows reading of multiple JsonElements from the specified reader
    asynchronously. The JSON data is parsed in lenient mode, see also JsonReader.setStrictness(Strictness).
    
    This class is conditionally thread-safe (see Item 70, Effective Java second edition). To
    properly use this class across multiple threads, you will need to add some external
    synchronization. For example:
    
    ```
    JsonStreamParser parser = new JsonStreamParser("['first'] {'second':10} 'third'");
    JsonElement element;
    synchronized (parser) {  // synchronize on an object shared by threads
      if (parser.hasNext()) {
        element = parser.next();
      }
    }
    ```

    Author(s)
    - Joel Leitch

    Since
    - 1.4
    """

    def __init__(self, json: str):
        """
        Arguments
        - json: The string containing JSON elements concatenated to each other.

        Since
        - 1.4
        """
        ...


    def __init__(self, reader: "Reader"):
        """
        Arguments
        - reader: The data stream containing JSON elements concatenated to each other.

        Since
        - 1.4
        """
        ...


    def next(self) -> "JsonElement":
        """
        Returns the next available JsonElement on the reader. Throws a NoSuchElementException if no element is available.

        Returns
        - the next available `JsonElement` on the reader.

        Raises
        - JsonParseException: if the incoming stream is malformed JSON.
        - NoSuchElementException: if no `JsonElement` is available.

        Since
        - 1.4
        """
        ...


    def hasNext(self) -> bool:
        """
        Returns True if a JsonElement is available on the input for consumption

        Returns
        - True if a JsonElement is available on the input, False otherwise

        Raises
        - JsonParseException: if the incoming stream is malformed JSON.

        Since
        - 1.4
        """
        ...


    def remove(self) -> None:
        """
        This optional Iterator method is not relevant for stream parsing and hence is not
        implemented.

        Since
        - 1.4
        """
        ...



"""
Python module generated from Java source file java.net.MalformedURLException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.net import *
from typing import Any, Callable, Iterable, Tuple


class MalformedURLException(IOException):
    """
    Thrown to indicate that a malformed URL has occurred. Either no
    legal protocol could be found in a specification string or the
    string could not be parsed.

    Author(s)
    - Arthur van Hoff

    Since
    - 1.0
    """

    def __init__(self):
        """
        Constructs a `MalformedURLException` with no detail message.
        """
        ...


    def __init__(self, msg: str):
        """
        Constructs a `MalformedURLException` with the
        specified detail message.

        Arguments
        - msg: the detail message.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.server.TabCompleteEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.command import CommandSender
from org.bukkit.event import Cancellable
from org.bukkit.event import Event
from org.bukkit.event import HandlerList
from org.bukkit.event.player import PlayerCommandSendEvent
from org.bukkit.event.server import *
from typing import Any, Callable, Iterable, Tuple


class TabCompleteEvent(Event, Cancellable):
    """
    Called when a CommandSender of any description (ie: player or
    console) attempts to tab complete.
    
    Note that due to client changes, if the sender is a Player, this event will
    only begin to fire once command arguments are specified, not commands
    themselves. Plugins wishing to remove commands from tab completion are
    advised to ensure the client does not have permission for the relevant
    commands, or use PlayerCommandSendEvent.
    """

    def __init__(self, sender: "CommandSender", buffer: str, completions: list[str]):
        ...


    def getSender(self) -> "CommandSender":
        """
        Get the sender completing this command.

        Returns
        - the CommandSender instance
        """
        ...


    def getBuffer(self) -> str:
        """
        Return the entire buffer which formed the basis of this completion.

        Returns
        - command buffer, as entered
        """
        ...


    def getCompletions(self) -> list[str]:
        """
        The list of completions which will be offered to the sender, in order.
        This list is mutable and reflects what will be offered.

        Returns
        - a list of offered completions
        """
        ...


    def setCompletions(self, completions: list[str]) -> None:
        """
        Set the completions offered, overriding any already set.

        Arguments
        - completions: the new completions
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.graph.IncidentEdgeSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.graph import *
from java.util import AbstractSet
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class IncidentEdgeSet(AbstractSet):
    """
    Abstract base class for an incident edges set that allows different implementations of AbstractSet.iterator().
    """

    def remove(self, o: "Object") -> bool:
        ...


    def size(self) -> int:
        ...


    def contains(self, obj: "Object") -> bool:
        ...



"""
Python module generated from Java source file java.util.Properties

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import BufferedWriter
from java.io import IOException
from java.io import InputStream
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import OutputStream
from java.io import OutputStreamWriter
from java.io import PrintStream
from java.io import PrintWriter
from java.io import Reader
from java.io import StreamCorruptedException
from java.io import UnsupportedEncodingException
from java.io import Writer
from java.nio.charset import Charset
from java.nio.charset import IllegalCharsetNameException
from java.nio.charset import UnsupportedCharsetException
from java.util import *
from java.util.concurrent import ConcurrentHashMap
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import Function
from jdk.internal.access import SharedSecrets
from jdk.internal.misc import Unsafe
from jdk.internal.util import ArraysSupport
from jdk.internal.util.xml import PropertiesDefaultHandler
from sun.nio.cs import ISO_8859_1
from sun.nio.cs import UTF_8
from typing import Any, Callable, Iterable, Tuple


class Properties(Hashtable):
    """
    The `Properties` class represents a persistent set of
    properties. The `Properties` can be saved to a stream
    or loaded from a stream. Each key and its corresponding value in
    the property list is a string.
    
    A property list can contain another property list as its
    "defaults"; this second property list is searched if
    the property key is not found in the original property list.
    
    Because `Properties` inherits from `Hashtable`, the
    `put` and `putAll` methods can be applied to a
    `Properties` object.  Their use is strongly discouraged as they
    allow the caller to insert entries whose keys or values are not
    `Strings`.  The `setProperty` method should be used
    instead.  If the `store` or `save` method is called
    on a "compromised" `Properties` object that contains a
    non-`String` key or value, the call will fail. Similarly,
    the call to the `propertyNames` or `list` method
    will fail if it is called on a "compromised" `Properties`
    object that contains a non-`String` key.
    
    
    The iterators returned by the `iterator` method of this class's
    "collection views" (that is, `entrySet()`, `keySet()`, and
    `values()`) may not fail-fast (unlike the Hashtable implementation).
    These iterators are guaranteed to traverse elements as they existed upon
    construction exactly once, and may (but are not guaranteed to) reflect any
    modifications subsequent to construction.
    
    The .load(java.io.Reader) load(Reader) `/`
    .store(java.io.Writer, java.lang.String) store(Writer, String)
    methods load and store properties from and to a character based stream
    in a simple line-oriented format specified below.
    
    The .load(java.io.InputStream) load(InputStream) `/`
    .store(java.io.OutputStream, java.lang.String) store(OutputStream, String)
    methods work the same way as the load(Reader)/store(Writer, String) pair, except
    the input/output stream is encoded in ISO 8859-1 character encoding.
    Characters that cannot be directly represented in this encoding can be written using
    Unicode escapes as defined in section 3.3 of
    <cite>The Java Language Specification</cite>;
    only a single 'u' character is allowed in an escape
    sequence.
    
     The .loadFromXML(InputStream) and .storeToXML(OutputStream, String, String) methods load and store properties
    in a simple XML format.  By default the UTF-8 character encoding is used,
    however a specific encoding may be specified if required. Implementations
    are required to support UTF-8 and UTF-16 and may support other encodings.
    An XML properties document has the following DOCTYPE declaration:
    
    ```
    &lt;!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd"&gt;
    ```
    Note that the system URI (http://java.sun.com/dtd/properties.dtd) is
    *not* accessed when exporting or importing properties; it merely
    serves as a string to uniquely identify the DTD, which is:
    ```
       &lt;?xml version="1.0" encoding="UTF-8"?&gt;
    
       &lt;!-- DTD for properties --&gt;
    
       &lt;!ELEMENT properties ( comment?, entry* ) &gt;
    
       &lt;!ATTLIST properties version CDATA #FIXED "1.0"&gt;
    
       &lt;!ELEMENT comment (#PCDATA) &gt;
    
       &lt;!ELEMENT entry (#PCDATA) &gt;
    
       &lt;!ATTLIST entry key CDATA #REQUIRED&gt;
    ```
    
    This class is thread-safe: multiple threads can share a single
    `Properties` object without the need for external synchronization.

    Author(s)
    - Xueming Shen

    Since
    - 1.0

    Unknown Tags
    - The `Properties` class does not inherit the concept of a load factor
    from its superclass, `Hashtable`.
    """

    def __init__(self):
        """
        Creates an empty property list with no default values.

        Unknown Tags
        - The initial capacity of a `Properties` object created
        with this constructor is unspecified.
        """
        ...


    def __init__(self, initialCapacity: int):
        """
        Creates an empty property list with no default values, and with an
        initial size accommodating the specified number of elements without the
        need to dynamically resize.

        Arguments
        - initialCapacity: the `Properties` will be sized to
                accommodate this many elements

        Raises
        - IllegalArgumentException: if the initial capacity is less than
                zero.
        """
        ...


    def __init__(self, defaults: "Properties"):
        """
        Creates an empty property list with the specified defaults.

        Arguments
        - defaults: the defaults.

        Unknown Tags
        - The initial capacity of a `Properties` object created
        with this constructor is unspecified.
        """
        ...


    def setProperty(self, key: str, value: str) -> "Object":
        """
        Calls the `Hashtable` method `put`. Provided for
        parallelism with the `getProperty` method. Enforces use of
        strings for property keys and values. The value returned is the
        result of the `Hashtable` call to `put`.

        Arguments
        - key: the key to be placed into this property list.
        - value: the value corresponding to `key`.

        Returns
        - the previous value of the specified key in this property
                    list, or `null` if it did not have one.

        See
        - .getProperty

        Since
        - 1.2
        """
        ...


    def load(self, reader: "Reader") -> None:
        """
        Reads a property list (key and element pairs) from the input
        character stream in a simple line-oriented format.
        
        Properties are processed in terms of lines. There are two
        kinds of line, *natural lines* and *logical lines*.
        A natural line is defined as a line of
        characters that is terminated either by a set of line terminator
        characters (`\n` or `\r` or `\r\n`)
        or by the end of the stream. A natural line may be either a blank line,
        a comment line, or hold all or some of a key-element pair. A logical
        line holds all the data of a key-element pair, which may be spread
        out across several adjacent natural lines by escaping
        the line terminator sequence with a backslash character
        `\`.  Note that a comment line cannot be extended
        in this manner; every natural line that is a comment must have
        its own comment indicator, as described below. Lines are read from
        input until the end of the stream is reached.
        
        
        A natural line that contains only white space characters is
        considered blank and is ignored.  A comment line has an ASCII
        `'.'` or `'!'` as its first non-white
        space character; comment lines are also ignored and do not
        encode key-element information.  In addition to line
        terminators, this format considers the characters space
        (`' '`, `'\u005Cu0020'`), tab
        (`'\t'`, `'\u005Cu0009'`), and form feed
        (`'\f'`, `'\u005Cu000C'`) to be white
        space.
        
        
        If a logical line is spread across several natural lines, the
        backslash escaping the line terminator sequence, the line
        terminator sequence, and any white space at the start of the
        following line have no affect on the key or element values.
        The remainder of the discussion of key and element parsing
        (when loading) will assume all the characters constituting
        the key and element appear on a single natural line after
        line continuation characters have been removed.  Note that
        it is *not* sufficient to only examine the character
        preceding a line terminator sequence to decide if the line
        terminator is escaped; there must be an odd number of
        contiguous backslashes for the line terminator to be escaped.
        Since the input is processed from left to right, a
        non-zero even number of 2*n* contiguous backslashes
        before a line terminator (or elsewhere) encodes *n*
        backslashes after escape processing.
        
        
        The key contains all of the characters in the line starting
        with the first non-white space character and up to, but not
        including, the first unescaped `'='`,
        `':'`, or white space character other than a line
        terminator. All of these key termination characters may be
        included in the key by escaping them with a preceding backslash
        character; for example,
        
        `\:\=`
        
        would be the two-character key `":="`.  Line
        terminator characters can be included using `\r` and
        `\n` escape sequences.  Any white space after the
        key is skipped; if the first non-white space character after
        the key is `'='` or `':'`, then it is
        ignored and any white space characters after it are also
        skipped.  All remaining characters on the line become part of
        the associated element string; if there are no remaining
        characters, the element is the empty string
        `""`.  Once the raw character sequences
        constituting the key and element are identified, escape
        processing is performed as described above.
        
        
        As an example, each of the following three lines specifies the key
        `"Truth"` and the associated element value
        `"Beauty"`:
        ```
        Truth = Beauty
         Truth:Beauty
        Truth                    :Beauty
        ```
        As another example, the following three lines specify a single
        property:
        ```
        fruits                           apple, banana, pear, \
                                         cantaloupe, watermelon, \
                                         kiwi, mango
        ```
        The key is `"fruits"` and the associated element is:
        ```"apple, banana, pear, cantaloupe, watermelon, kiwi, mango"```
        Note that a space appears before each `\` so that a space
        will appear after each comma in the final result; the `\`,
        line terminator, and leading white space on the continuation line are
        merely discarded and are *not* replaced by one or more other
        characters.
        
        As a third example, the line:
        ```cheeses
        ```
        specifies that the key is `"cheeses"` and the associated
        element is the empty string `""`.
        
        <a id="unicodeescapes"></a>
        Characters in keys and elements can be represented in escape
        sequences similar to those used for character and string literals
        (see sections 3.3 and 3.10.6 of
        <cite>The Java Language Specification</cite>).
        
        The differences from the character escape sequences and Unicode
        escapes used for characters and strings are:
        
        
        -  Octal escapes are not recognized.
        
        -  The character sequence `\b` does *not*
        represent a backspace character.
        
        -  The method does not treat a backslash character,
        `\`, before a non-valid escape character as an
        error; the backslash is silently dropped.  For example, in a
        Java string the sequence `"\z"` would cause a
        compile time error.  In contrast, this method silently drops
        the backslash.  Therefore, this method treats the two character
        sequence `"\b"` as equivalent to the single
        character `'b'`.
        
        -  Escapes are not necessary for single and double quotes;
        however, by the rule above, single and double quote characters
        preceded by a backslash still yield single and double quote
        characters, respectively.
        
        -  Only a single 'u' character is allowed in a Unicode escape
        sequence.
        
        
        
        The specified stream remains open after this method returns.

        Arguments
        - reader: the input character stream.

        Raises
        - IOException: if an error occurred when reading from the
                 input stream.
        - IllegalArgumentException: if a malformed Unicode escape
                 appears in the input.
        - NullPointerException: if `reader` is null.

        Since
        - 1.6
        """
        ...


    def load(self, inStream: "InputStream") -> None:
        """
        Reads a property list (key and element pairs) from the input
        byte stream. The input stream is in a simple line-oriented
        format as specified in
        .load(java.io.Reader) load(Reader) and is assumed to use
        the ISO 8859-1 character encoding; that is each byte is one Latin1
        character. Characters not in Latin1, and certain special characters,
        are represented in keys and elements using Unicode escapes as defined in
        section 3.3 of
        <cite>The Java Language Specification</cite>.
        
        The specified stream remains open after this method returns.

        Arguments
        - inStream: the input stream.

        Raises
        - IOException: if an error occurred when reading from the
                    input stream.
        - IllegalArgumentException: if the input stream contains a
                    malformed Unicode escape sequence.
        - NullPointerException: if `inStream` is null.

        Since
        - 1.2
        """
        ...


    def save(self, out: "OutputStream", comments: str) -> None:
        """
        Calls the `store(OutputStream out, String comments)` method
        and suppresses IOExceptions that were thrown.

        Arguments
        - out: an output stream.
        - comments: a description of the property list.

        Raises
        - ClassCastException: if this `Properties` object
                    contains any keys or values that are not
                    `Strings`.

        Deprecated
        - This method does not throw an IOException if an I/O error
        occurs while saving the property list.  The preferred way to save a
        properties list is via the `store(OutputStream out,
        String comments)` method or the
        `storeToXML(OutputStream os, String comment)` method.
        """
        ...


    def store(self, writer: "Writer", comments: str) -> None:
        """
        Writes this property list (key and element pairs) in this
        `Properties` table to the output character stream in a
        format suitable for using the .load(java.io.Reader) load(Reader)
        method.
        
        Properties from the defaults table of this `Properties`
        table (if any) are *not* written out by this method.
        
        If the comments argument is not null, then an ASCII `.`
        character, the comments string, and a line separator are first written
        to the output stream. Thus, the `comments` can serve as an
        identifying comment. Any one of a line feed ('\n'), a carriage
        return ('\r'), or a carriage return followed immediately by a line feed
        in comments is replaced by a line separator generated by the `Writer`
        and if the next character in comments is not character `.` or
        character `!` then an ASCII `.` is written out
        after that line separator.
        
        Next, a comment line is always written, consisting of an ASCII
        `.` character, the current date and time (as if produced
        by the `toString` method of `Date` for the
        current time), and a line separator as generated by the `Writer`.
        
        Then every entry in this `Properties` table is
        written out, one per line. For each entry the key string is
        written, then an ASCII `=`, then the associated
        element string. For the key, all space characters are
        written with a preceding `\` character.  For the
        element, leading space characters, but not embedded or trailing
        space characters, are written with a preceding `\`
        character. The key and element characters `.`,
        `!`, `=`, and `:` are written
        with a preceding backslash to ensure that they are properly loaded.
        
        After the entries have been written, the output stream is flushed.
        The output stream remains open after this method returns.

        Arguments
        - writer: an output character stream writer.
        - comments: a description of the property list.

        Raises
        - IOException: if writing this property list to the specified
                    output stream throws an `IOException`.
        - ClassCastException: if this `Properties` object
                    contains any keys or values that are not `Strings`.
        - NullPointerException: if `writer` is null.

        Since
        - 1.6
        """
        ...


    def store(self, out: "OutputStream", comments: str) -> None:
        """
        Writes this property list (key and element pairs) in this
        `Properties` table to the output stream in a format suitable
        for loading into a `Properties` table using the
        .load(InputStream) load(InputStream) method.
        
        Properties from the defaults table of this `Properties`
        table (if any) are *not* written out by this method.
        
        This method outputs the comments, properties keys and values in
        the same format as specified in
        .store(java.io.Writer, java.lang.String) store(Writer),
        with the following differences:
        
        - The stream is written using the ISO 8859-1 character encoding.
        
        - Characters not in Latin-1 in the comments are written as
        `\u005Cu`*xxxx* for their appropriate unicode
        hexadecimal value *xxxx*.
        
        - Characters less than `\u005Cu0020` and characters greater
        than `\u005Cu007E` in property keys or values are written
        as `\u005Cu`*xxxx* for the appropriate hexadecimal
        value *xxxx*.
        
        
        After the entries have been written, the output stream is flushed.
        The output stream remains open after this method returns.

        Arguments
        - out: an output stream.
        - comments: a description of the property list.

        Raises
        - IOException: if writing this property list to the specified
                    output stream throws an `IOException`.
        - ClassCastException: if this `Properties` object
                    contains any keys or values that are not `Strings`.
        - NullPointerException: if `out` is null.

        Since
        - 1.2
        """
        ...


    def loadFromXML(self, in: "InputStream") -> None:
        """
        Loads all of the properties represented by the XML document on the
        specified input stream into this properties table.
        
        The XML document must have the following DOCTYPE declaration:
        ```
        &lt;!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd"&gt;
        ```
        Furthermore, the document must satisfy the properties DTD described
        above.
        
         An implementation is required to read XML documents that use the
        "`UTF-8`" or "`UTF-16`" encoding. An implementation may
        support additional encodings.
        
        The specified stream is closed after this method returns.

        Arguments
        - in: the input stream from which to read the XML document.

        Raises
        - IOException: if reading from the specified input stream
                results in an `IOException`.
        - java.io.UnsupportedEncodingException: if the document's encoding
                declaration can be read and it specifies an encoding that is not
                supported
        - InvalidPropertiesFormatException: Data on input stream does not
                constitute a valid XML document with the mandated document type.
        - NullPointerException: if `in` is null.

        See
        - <a href="http://www.w3.org/TR/REC-xml/.charencoding">Character
                Encoding in Entities</a>

        Since
        - 1.5
        """
        ...


    def storeToXML(self, os: "OutputStream", comment: str) -> None:
        """
        Emits an XML document representing all of the properties contained
        in this table.
        
         An invocation of this method of the form `props.storeToXML(os,
        comment)` behaves in exactly the same way as the invocation
        `props.storeToXML(os, comment, "UTF-8");`.

        Arguments
        - os: the output stream on which to emit the XML document.
        - comment: a description of the property list, or `null`
               if no comment is desired.

        Raises
        - IOException: if writing to the specified output stream
                results in an `IOException`.
        - NullPointerException: if `os` is null.
        - ClassCastException: if this `Properties` object
                contains any keys or values that are not
                `Strings`.

        See
        - .loadFromXML(InputStream)

        Since
        - 1.5
        """
        ...


    def storeToXML(self, os: "OutputStream", comment: str, encoding: str) -> None:
        """
        Emits an XML document representing all of the properties contained
        in this table, using the specified encoding.
        
        The XML document will have the following DOCTYPE declaration:
        ```
        &lt;!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd"&gt;
        ```
        
        If the specified comment is `null` then no comment
        will be stored in the document.
        
         An implementation is required to support writing of XML documents
        that use the "`UTF-8`" or "`UTF-16`" encoding. An
        implementation may support additional encodings.
        
        The specified stream remains open after this method returns.
        
        This method behaves the same as
        .storeToXML(OutputStream os, String comment, Charset charset)
        except that it will java.nio.charset.Charset.forName look up the charset
        using the given encoding name.

        Arguments
        - os: the output stream on which to emit the XML document.
        - comment: a description of the property list, or `null`
                         if no comment is desired.
        - encoding: the name of a supported
                         <a href="../lang/package-summary.html#charenc">
                         character encoding</a>

        Raises
        - IOException: if writing to the specified output stream
                results in an `IOException`.
        - java.io.UnsupportedEncodingException: if the encoding is not
                supported by the implementation.
        - NullPointerException: if `os` is `null`,
                or if `encoding` is `null`.
        - ClassCastException: if this `Properties` object
                contains any keys or values that are not `Strings`.

        See
        - <a href="http://www.w3.org/TR/REC-xml/.charencoding">Character
                Encoding in Entities</a>

        Since
        - 1.5
        """
        ...


    def storeToXML(self, os: "OutputStream", comment: str, charset: "Charset") -> None:
        """
        Emits an XML document representing all of the properties contained
        in this table, using the specified encoding.
        
        The XML document will have the following DOCTYPE declaration:
        ```
        &lt;!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd"&gt;
        ```
        
        If the specified comment is `null` then no comment
        will be stored in the document.
        
         An implementation is required to support writing of XML documents
        that use the "`UTF-8`" or "`UTF-16`" encoding. An
        implementation may support additional encodings.
        
         Unmappable characters for the specified charset will be encoded as
        numeric character references.
        
        The specified stream remains open after this method returns.

        Arguments
        - os: the output stream on which to emit the XML document.
        - comment: a description of the property list, or `null`
                         if no comment is desired.
        - charset: the charset

        Raises
        - IOException: if writing to the specified output stream
                results in an `IOException`.
        - NullPointerException: if `os` or `charset` is `null`.
        - ClassCastException: if this `Properties` object
                contains any keys or values that are not `Strings`.

        See
        - <a href="http://www.w3.org/TR/REC-xml/.charencoding">Character
                Encoding in Entities</a>

        Since
        - 10
        """
        ...


    def getProperty(self, key: str) -> str:
        """
        Searches for the property with the specified key in this property list.
        If the key is not found in this property list, the default property list,
        and its defaults, recursively, are then checked. The method returns
        `null` if the property is not found.

        Arguments
        - key: the property key.

        Returns
        - the value in this property list with the specified key value.

        See
        - .defaults
        """
        ...


    def getProperty(self, key: str, defaultValue: str) -> str:
        """
        Searches for the property with the specified key in this property list.
        If the key is not found in this property list, the default property list,
        and its defaults, recursively, are then checked. The method returns the
        default value argument if the property is not found.

        Arguments
        - key: the hashtable key.
        - defaultValue: a default value.

        Returns
        - the value in this property list with the specified key value.

        See
        - .defaults
        """
        ...


    def propertyNames(self) -> "Enumeration"[Any]:
        """
        Returns an enumeration of all the keys in this property list,
        including distinct keys in the default property list if a key
        of the same name has not already been found from the main
        properties list.

        Returns
        - an enumeration of all the keys in this property list, including
                 the keys in the default property list.

        Raises
        - ClassCastException: if any key in this property list
                 is not a string.

        See
        - .stringPropertyNames
        """
        ...


    def stringPropertyNames(self) -> set[str]:
        """
        Returns an unmodifiable set of keys from this property list
        where the key and its corresponding value are strings,
        including distinct keys in the default property list if a key
        of the same name has not already been found from the main
        properties list.  Properties whose key or value is not
        of type `String` are omitted.
        
        The returned set is not backed by this `Properties` object.
        Changes to this `Properties` object are not reflected in the
        returned set.

        Returns
        - an unmodifiable set of keys in this property list where
                 the key and its corresponding value are strings,
                 including the keys in the default property list.

        See
        - java.util.Properties.defaults

        Since
        - 1.6
        """
        ...


    def list(self, out: "PrintStream") -> None:
        """
        Prints this property list out to the specified output stream.
        This method is useful for debugging.

        Arguments
        - out: an output stream.

        Raises
        - ClassCastException: if any key in this property list
                 is not a string.
        """
        ...


    def list(self, out: "PrintWriter") -> None:
        ...


    def size(self) -> int:
        ...


    def isEmpty(self) -> bool:
        ...


    def keys(self) -> "Enumeration"["Object"]:
        ...


    def elements(self) -> "Enumeration"["Object"]:
        ...


    def contains(self, value: "Object") -> bool:
        ...


    def containsValue(self, value: "Object") -> bool:
        ...


    def containsKey(self, key: "Object") -> bool:
        ...


    def get(self, key: "Object") -> "Object":
        ...


    def put(self, key: "Object", value: "Object") -> "Object":
        ...


    def remove(self, key: "Object") -> "Object":
        ...


    def putAll(self, t: dict[Any, Any]) -> None:
        ...


    def clear(self) -> None:
        ...


    def toString(self) -> str:
        ...


    def keySet(self) -> set["Object"]:
        ...


    def values(self) -> Iterable["Object"]:
        ...


    def entrySet(self) -> set["Map.Entry"["Object", "Object"]]:
        ...


    def equals(self, o: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def getOrDefault(self, key: "Object", defaultValue: "Object") -> "Object":
        ...


    def forEach(self, action: "BiConsumer"["Object", "Object"]) -> None:
        ...


    def replaceAll(self, function: "BiFunction"["Object", "Object", Any]) -> None:
        ...


    def putIfAbsent(self, key: "Object", value: "Object") -> "Object":
        ...


    def remove(self, key: "Object", value: "Object") -> bool:
        ...


    def replace(self, key: "Object", oldValue: "Object", newValue: "Object") -> bool:
        ...


    def replace(self, key: "Object", value: "Object") -> "Object":
        ...


    def computeIfAbsent(self, key: "Object", mappingFunction: "Function"["Object", Any]) -> "Object":
        ...


    def computeIfPresent(self, key: "Object", remappingFunction: "BiFunction"["Object", "Object", Any]) -> "Object":
        ...


    def compute(self, key: "Object", remappingFunction: "BiFunction"["Object", "Object", Any]) -> "Object":
        ...


    def merge(self, key: "Object", value: "Object", remappingFunction: "BiFunction"["Object", "Object", Any]) -> "Object":
        ...


    def clone(self) -> "Object":
        ...
